#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_firewall_vip64
short_description: Configure IPv6 to IPv4 virtual IPs in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure firewall feature and vip64 category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    firewall_vip64:
        description:
            - Configure IPv6 to IPv4 virtual IPs.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            arp-reply:
                description:
                    - Enable ARP reply.
                choices:
                    - disable
                    - enable
            color:
                description:
                    - Color of icon on the GUI.
            comment:
                description:
                    - Comment.
            extip:
                description:
                    - Start-external-IP [-end-external-IP].
            extport:
                description:
                    - External service port.
            id:
                description:
                    - Custom defined id.
            ldb-method:
                description:
                    - Load balance method.
                choices:
                    - static
                    - round-robin
                    - weighted
                    - least-session
                    - least-rtt
                    - first-alive
            mappedip:
                description:
                    - Start-mapped-IP [-end-mapped-IP].
            mappedport:
                description:
                    - Mapped service port.
            monitor:
                description:
                    - Health monitors.
                suboptions:
                    name:
                        description:
                            - Health monitor name. Source firewall.ldb-monitor.name.
                        required: true
            name:
                description:
                    - VIP64 name.
                required: true
            portforward:
                description:
                    - Enable port forwarding.
                choices:
                    - disable
                    - enable
            protocol:
                description:
                    - Mapped port protocol.
                choices:
                    - tcp
                    - udp
            realservers:
                description:
                    - Real servers.
                suboptions:
                    client-ip:
                        description:
                            - Restrict server to a client IP in this range.
                    healthcheck:
                        description:
                            - Per server health check.
                        choices:
                            - disable
                            - enable
                            - vip
                    holddown-interval:
                        description:
                            - Hold down interval.
                    id:
                        description:
                            - Real server ID.
                        required: true
                    ip:
                        description:
                            - Mapped server IP.
                    max-connections:
                        description:
                            - Maximum number of connections allowed to server.
                    monitor:
                        description:
                            - Health monitors. Source firewall.ldb-monitor.name.
                    port:
                        description:
                            - Mapped server port.
                    status:
                        description:
                            - Server administrative status.
                        choices:
                            - active
                            - standby
                            - disable
                    weight:
                        description:
                            - weight
            server-type:
                description:
                    - Server type.
                choices:
                    - http
                    - tcp
                    - udp
                    - ip
            src-filter:
                description:
                    - "Source IP6 filter (x:x:x:x:x:x:x:x/x)."
                suboptions:
                    range:
                        description:
                            - Src-filter range.
                        required: true
            type:
                description:
                    - "VIP type: static NAT or server load balance."
                choices:
                    - static-nat
                    - server-load-balance
            uuid:
                description:
                    - Universally Unique Identifier (UUID; automatically assigned but can be manually reset).
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPv6 to IPv4 virtual IPs.
    fortios_firewall_vip64:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      firewall_vip64:
        state: "present"
        arp-reply: "disable"
        color: "4"
        comment: "Comment."
        extip: "<your_own_value>"
        extport: "<your_own_value>"
        id:  "8"
        ldb-method: "static"
        mappedip: "<your_own_value>"
        mappedport: "<your_own_value>"
        monitor:
         -
            name: "default_name_13 (source firewall.ldb-monitor.name)"
        name: "default_name_14"
        portforward: "disable"
        protocol: "tcp"
        realservers:
         -
            client-ip: "<your_own_value>"
            healthcheck: "disable"
            holddown-interval: "20"
            id:  "21"
            ip: "<your_own_value>"
            max-connections: "23"
            monitor: "<your_own_value> (source firewall.ldb-monitor.name)"
            port: "25"
            status: "active"
            weight: "27"
        server-type: "http"
        src-filter:
         -
            range: "<your_own_value>"
        type: "static-nat"
        uuid: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_firewall_vip64_data(json):
    option_list = ['arp-reply', 'color', 'comment',
                   'extip', 'extport', 'id',
                   'ldb-method', 'mappedip', 'mappedport',
                   'monitor', 'name', 'portforward',
                   'protocol', 'realservers', 'server-type',
                   'src-filter', 'type', 'uuid']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def firewall_vip64(data, fos):
    vdom = data['vdom']
    firewall_vip64_data = data['firewall_vip64']
    filtered_data = filter_firewall_vip64_data(firewall_vip64_data)
    if firewall_vip64_data['state'] == "present":
        return fos.set('firewall',
                       'vip64',
                       data=filtered_data,
                       vdom=vdom)

    elif firewall_vip64_data['state'] == "absent":
        return fos.delete('firewall',
                          'vip64',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_firewall(data, fos):
    login(data)

    methodlist = ['firewall_vip64']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "firewall_vip64": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "arp-reply": {"required": False, "type": "str",
                              "choices": ["disable", "enable"]},
                "color": {"required": False, "type": "int"},
                "comment": {"required": False, "type": "str"},
                "extip": {"required": False, "type": "str"},
                "extport": {"required": False, "type": "str"},
                "id": {"required": False, "type": "int"},
                "ldb-method": {"required": False, "type": "str",
                               "choices": ["static", "round-robin", "weighted",
                                           "least-session", "least-rtt", "first-alive"]},
                "mappedip": {"required": False, "type": "str"},
                "mappedport": {"required": False, "type": "str"},
                "monitor": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "name": {"required": True, "type": "str"},
                "portforward": {"required": False, "type": "str",
                                "choices": ["disable", "enable"]},
                "protocol": {"required": False, "type": "str",
                             "choices": ["tcp", "udp"]},
                "realservers": {"required": False, "type": "list",
                                "options": {
                                    "client-ip": {"required": False, "type": "str"},
                                    "healthcheck": {"required": False, "type": "str",
                                                    "choices": ["disable", "enable", "vip"]},
                                    "holddown-interval": {"required": False, "type": "int"},
                                    "id": {"required": True, "type": "int"},
                                    "ip": {"required": False, "type": "str"},
                                    "max-connections": {"required": False, "type": "int"},
                                    "monitor": {"required": False, "type": "str"},
                                    "port": {"required": False, "type": "int"},
                                    "status": {"required": False, "type": "str",
                                               "choices": ["active", "standby", "disable"]},
                                    "weight": {"required": False, "type": "int"}
                                }},
                "server-type": {"required": False, "type": "str",
                                "choices": ["http", "tcp", "udp",
                                            "ip"]},
                "src-filter": {"required": False, "type": "list",
                               "options": {
                                   "range": {"required": True, "type": "str"}
                               }},
                "type": {"required": False, "type": "str",
                         "choices": ["static-nat", "server-load-balance"]},
                "uuid": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_firewall(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_wireless_controller_wtp_profile
short_description: Configure WTP profiles or FortiAP profiles that define radio settings for manageable FortiAP platforms in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify wireless_controller feature and wtp_profile category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    wireless_controller_wtp_profile:
        description:
            - Configure WTP profiles or FortiAP profiles that define radio settings for manageable FortiAP platforms.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            allowaccess:
                description:
                    - Control management access to the managed WTP, FortiAP, or AP. Separate entries with a space.
                choices:
                    - telnet
                    - http
                    - https
                    - ssh
            ap-country:
                description:
                    - Country in which this WTP, FortiAP or AP will operate (default = US).
                choices:
                    - NA
                    - AL
                    - DZ
                    - AO
                    - AR
                    - AM
                    - AU
                    - AT
                    - AZ
                    - BH
                    - BD
                    - BB
                    - BY
                    - BE
                    - BZ
                    - BO
                    - BA
                    - BR
                    - BN
                    - BG
                    - KH
                    - CL
                    - CN
                    - CO
                    - CR
                    - HR
                    - CY
                    - CZ
                    - DK
                    - DO
                    - EC
                    - EG
                    - SV
                    - EE
                    - FI
                    - FR
                    - GE
                    - DE
                    - GR
                    - GL
                    - GD
                    - GU
                    - GT
                    - HT
                    - HN
                    - HK
                    - HU
                    - IS
                    - IN
                    - ID
                    - IR
                    - IE
                    - IL
                    - IT
                    - JM
                    - JO
                    - KZ
                    - KE
                    - KP
                    - KR
                    - KW
                    - LV
                    - LB
                    - LI
                    - LT
                    - LU
                    - MO
                    - MK
                    - MY
                    - MT
                    - MX
                    - MC
                    - MA
                    - MZ
                    - MM
                    - NP
                    - NL
                    - AN
                    - AW
                    - NZ
                    - NO
                    - OM
                    - PK
                    - PA
                    - PG
                    - PY
                    - PE
                    - PH
                    - PL
                    - PT
                    - PR
                    - QA
                    - RO
                    - RU
                    - RW
                    - SA
                    - RS
                    - ME
                    - SG
                    - SK
                    - SI
                    - ZA
                    - ES
                    - LK
                    - SE
                    - SD
                    - CH
                    - SY
                    - TW
                    - TZ
                    - TH
                    - TT
                    - TN
                    - TR
                    - AE
                    - UA
                    - GB
                    - US
                    - PS
                    - UY
                    - UZ
                    - VE
                    - VN
                    - YE
                    - ZB
                    - ZW
                    - JP
                    - CA
            ble-profile:
                description:
                    - Bluetooth Low Energy profile name. Source wireless-controller.ble-profile.name.
            comment:
                description:
                    - Comment.
            control-message-offload:
                description:
                    - Enable/disable CAPWAP control message data channel offload.
                choices:
                    - ebp-frame
                    - aeroscout-tag
                    - ap-list
                    - sta-list
                    - sta-cap-list
                    - stats
                    - aeroscout-mu
            deny-mac-list:
                description:
                    - List of MAC addresses that are denied access to this WTP, FortiAP, or AP.
                suboptions:
                    id:
                        description:
                            - ID.
                        required: true
                    mac:
                        description:
                            - A WiFi device with this MAC address is denied access to this WTP, FortiAP or AP.
            dtls-in-kernel:
                description:
                    - Enable/disable data channel DTLS in kernel.
                choices:
                    - enable
                    - disable
            dtls-policy:
                description:
                    - WTP data channel DTLS policy (default = clear-text).
                choices:
                    - clear-text
                    - dtls-enabled
                    - ipsec-vpn
            energy-efficient-ethernet:
                description:
                    - Enable/disable use of energy efficient Ethernet on WTP.
                choices:
                    - enable
                    - disable
            ext-info-enable:
                description:
                    - Enable/disable station/VAP/radio extension information.
                choices:
                    - enable
                    - disable
            handoff-roaming:
                description:
                    - Enable/disable client load balancing during roaming to avoid roaming delay (default = disable).
                choices:
                    - enable
                    - disable
            handoff-rssi:
                description:
                    - Minimum received signal strength indicator (RSSI) value for handoff (20 - 30, default = 25).
            handoff-sta-thresh:
                description:
                    - Threshold value for AP handoff (5 - 35, default = 30).
            ip-fragment-preventing:
                description:
                    - Select how to prevent IP fragmentation for CAPWAP tunneled control and data packets (default = tcp-mss-adjust).
                choices:
                    - tcp-mss-adjust
                    - icmp-unreachable
            lan:
                description:
                    - WTP LAN port mapping.
                suboptions:
                    port-mode:
                        description:
                            - LAN port mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port-ssid:
                        description:
                            - Bridge LAN port to SSID. Source wireless-controller.vap.name.
                    port1-mode:
                        description:
                            - LAN port 1 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port1-ssid:
                        description:
                            - Bridge LAN port 1 to SSID. Source wireless-controller.vap.name.
                    port2-mode:
                        description:
                            - LAN port 2 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port2-ssid:
                        description:
                            - Bridge LAN port 2 to SSID. Source wireless-controller.vap.name.
                    port3-mode:
                        description:
                            - LAN port 3 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port3-ssid:
                        description:
                            - Bridge LAN port 3 to SSID. Source wireless-controller.vap.name.
                    port4-mode:
                        description:
                            - LAN port 4 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port4-ssid:
                        description:
                            - Bridge LAN port 4 to SSID. Source wireless-controller.vap.name.
                    port5-mode:
                        description:
                            - LAN port 5 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port5-ssid:
                        description:
                            - Bridge LAN port 5 to SSID. Source wireless-controller.vap.name.
                    port6-mode:
                        description:
                            - LAN port 6 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port6-ssid:
                        description:
                            - Bridge LAN port 6 to SSID. Source wireless-controller.vap.name.
                    port7-mode:
                        description:
                            - LAN port 7 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port7-ssid:
                        description:
                            - Bridge LAN port 7 to SSID. Source wireless-controller.vap.name.
                    port8-mode:
                        description:
                            - LAN port 8 mode.
                        choices:
                            - offline
                            - nat-to-wan
                            - bridge-to-wan
                            - bridge-to-ssid
                    port8-ssid:
                        description:
                            - Bridge LAN port 8 to SSID. Source wireless-controller.vap.name.
            lbs:
                description:
                    - Set various location based service (LBS) options.
                suboptions:
                    aeroscout:
                        description:
                            - Enable/disable AeroScout Real Time Location Service (RTLS) support.
                        choices:
                            - enable
                            - disable
                    aeroscout-ap-mac:
                        description:
                            - Use BSSID or board MAC address as AP MAC address in the Aeroscout AP message.
                        choices:
                            - bssid
                            - board-mac
                    aeroscout-mmu-report:
                        description:
                            - Enable/disable MU compounded report.
                        choices:
                            - enable
                            - disable
                    aeroscout-mu:
                        description:
                            - Enable/disable AeroScout support.
                        choices:
                            - enable
                            - disable
                    aeroscout-mu-factor:
                        description:
                            - AeroScout Mobile Unit (MU) mode dilution factor (default = 20).
                    aeroscout-mu-timeout:
                        description:
                            - AeroScout MU mode timeout (0 - 65535 sec, default = 5).
                    aeroscout-server-ip:
                        description:
                            - IP address of AeroScout server.
                    aeroscout-server-port:
                        description:
                            - AeroScout server UDP listening port.
                    ekahau-blink-mode:
                        description:
                            - Enable/disable Ekahua blink mode (also called AiRISTA Flow Blink Mode) to find the location of devices connected to a wireless
                               LAN (default = disable).
                        choices:
                            - enable
                            - disable
                    ekahau-tag:
                        description:
                            - WiFi frame MAC address or WiFi Tag.
                    erc-server-ip:
                        description:
                            - IP address of Ekahua RTLS Controller (ERC).
                    erc-server-port:
                        description:
                            - Ekahua RTLS Controller (ERC) UDP listening port.
                    fortipresence:
                        description:
                            - Enable/disable FortiPresence to monitor the location and activity of WiFi clients even if they don't connect to this WiFi
                               network (default = disable).
                        choices:
                            - foreign
                            - both
                            - disable
                    fortipresence-frequency:
                        description:
                            - FortiPresence report transmit frequency (5 - 65535 sec, default = 30).
                    fortipresence-port:
                        description:
                            - FortiPresence server UDP listening port (default = 3000).
                    fortipresence-project:
                        description:
                            - FortiPresence project name (max. 16 characters, default = fortipresence).
                    fortipresence-rogue:
                        description:
                            - Enable/disable FortiPresence finding and reporting rogue APs.
                        choices:
                            - enable
                            - disable
                    fortipresence-secret:
                        description:
                            - FortiPresence secret password (max. 16 characters).
                    fortipresence-server:
                        description:
                            - FortiPresence server IP address.
                    fortipresence-unassoc:
                        description:
                            - Enable/disable FortiPresence finding and reporting unassociated stations.
                        choices:
                            - enable
                            - disable
                    station-locate:
                        description:
                            - Enable/disable client station locating services for all clients, whether associated or not (default = disable).
                        choices:
                            - enable
                            - disable
            led-schedules:
                description:
                    - Recurring firewall schedules for illuminating LEDs on the FortiAP. If led-state is enabled, LEDs will be visible when at least one of
                       the schedules is valid. Separate multiple schedule names with a space.
                suboptions:
                    name:
                        description:
                            - LED schedule name. Source firewall.schedule.group.name firewall.schedule.recurring.name.
                        required: true
            led-state:
                description:
                    - Enable/disable use of LEDs on WTP (default = disable).
                choices:
                    - enable
                    - disable
            lldp:
                description:
                    - Enable/disable Link Layer Discovery Protocol (LLDP) for the WTP, FortiAP, or AP (default = disable).
                choices:
                    - enable
                    - disable
            login-passwd:
                description:
                    - Set the managed WTP, FortiAP, or AP's administrator password.
            login-passwd-change:
                description:
                    - Change or reset the administrator password of a managed WTP, FortiAP or AP (yes, default, or no, default = no).
                choices:
                    - yes
                    - default
                    - no
            max-clients:
                description:
                    - Maximum number of stations (STAs) supported by the WTP (default = 0, meaning no client limitation).
            name:
                description:
                    - WTP (or FortiAP or AP) profile name.
                required: true
            platform:
                description:
                    - WTP, FortiAP, or AP platform.
                suboptions:
                    type:
                        description:
                            - WTP, FortiAP or AP platform type. There are built-in WTP profiles for all supported FortiAP models. You can select a built-in
                               profile and customize it or create a new profile.
                        choices:
                            - AP-11N
                            - 220B
                            - 210B
                            - 222B
                            - 112B
                            - 320B
                            - 11C
                            - 14C
                            - 223B
                            - 28C
                            - 320C
                            - 221C
                            - 25D
                            - 222C
                            - 224D
                            - 214B
                            - 21D
                            - 24D
                            - 112D
                            - 223C
                            - 321C
                            - C220C
                            - C225C
                            - C23JD
                            - C24JE
                            - S321C
                            - S322C
                            - S323C
                            - S311C
                            - S313C
                            - S321CR
                            - S322CR
                            - S323CR
                            - S421E
                            - S422E
                            - S423E
                            - 421E
                            - 423E
                            - 221E
                            - 222E
                            - 223E
                            - 224E
                            - S221E
                            - S223E
                            - U421E
                            - U422EV
                            - U423E
                            - U221EV
                            - U223EV
                            - U24JEV
                            - U321EV
                            - U323EV
            poe-mode:
                description:
                    - Set the WTP, FortiAP, or AP's PoE mode.
                choices:
                    - auto
                    - 8023af
                    - 8023at
                    - power-adapter
            radio-1:
                description:
                    - Configuration options for radio 1.
                suboptions:
                    amsdu:
                        description:
                            - Enable/disable 802.11n AMSDU support. AMSDU can improve performance if supported by your WiFi clients (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-handoff:
                        description:
                            - Enable/disable AP handoff of clients to other APs (default = disable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-addr:
                        description:
                            - MAC address to monitor.
                    ap-sniffer-bufsize:
                        description:
                            - Sniffer buffer size (1 - 32 MB, default = 16).
                    ap-sniffer-chan:
                        description:
                            - Channel on which to operate the sniffer (default = 6).
                    ap-sniffer-ctl:
                        description:
                            - Enable/disable sniffer on WiFi control frame (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-data:
                        description:
                            - Enable/disable sniffer on WiFi data frame (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-mgmt-beacon:
                        description:
                            - Enable/disable sniffer on WiFi management Beacon frames (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-mgmt-other:
                        description:
                            - Enable/disable sniffer on WiFi management other frames  (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-mgmt-probe:
                        description:
                            - Enable/disable sniffer on WiFi management probe frames (default = enable).
                        choices:
                            - enable
                            - disable
                    auto-power-high:
                        description:
                            - Automatic transmit power high limit in dBm (the actual range of transmit power depends on the AP platform type).
                    auto-power-level:
                        description:
                            - Enable/disable automatic power-level adjustment to prevent co-channel interference (default = disable).
                        choices:
                            - enable
                            - disable
                    auto-power-low:
                        description:
                            - Automatic transmission power low limit in dBm (the actual range of transmit power depends on the AP platform type).
                    band:
                        description:
                            - WiFi band that Radio 1 operates on.
                        choices:
                            - 802.11a
                            - 802.11b
                            - 802.11g
                            - 802.11n
                            - 802.11n-5G
                            - 802.11ac
                            - 802.11n,g-only
                            - 802.11g-only
                            - 802.11n-only
                            - 802.11n-5G-only
                            - 802.11ac,n-only
                            - 802.11ac-only
                    bandwidth-admission-control:
                        description:
                            - Enable/disable WiFi multimedia (WMM) bandwidth admission control to optimize WiFi bandwidth use. A request to join the wireless
                               network is only allowed if the access point has enough bandwidth to support it.
                        choices:
                            - enable
                            - disable
                    bandwidth-capacity:
                        description:
                            - Maximum bandwidth capacity allowed (1 - 600000 Kbps, default = 2000).
                    beacon-interval:
                        description:
                            - Beacon interval. The time between beacon frames in msec (the actual range of beacon interval depends on the AP platform type,
                               default = 100).
                    call-admission-control:
                        description:
                            - Enable/disable WiFi multimedia (WMM) call admission control to optimize WiFi bandwidth use for VoIP calls. New VoIP calls are
                               only accepted if there is enough bandwidth available to support them.
                        choices:
                            - enable
                            - disable
                    call-capacity:
                        description:
                            - Maximum number of Voice over WLAN (VoWLAN) phones supported by the radio (0 - 60, default = 10).
                    channel:
                        description:
                            - Selected list of wireless radio channels.
                        suboptions:
                            chan:
                                description:
                                    - Channel number.
                                required: true
                    channel-bonding:
                        description:
                            - "Channel bandwidth: 80, 40, or 20MHz. Channels may use both 20 and 40 by enabling coexistence."
                        choices:
                            - 80MHz
                            - 40MHz
                            - 20MHz
                    channel-utilization:
                        description:
                            - Enable/disable measuring channel utilization.
                        choices:
                            - enable
                            - disable
                    coexistence:
                        description:
                            - Enable/disable allowing both HT20 and HT40 on the same radio (default = enable).
                        choices:
                            - enable
                            - disable
                    darrp:
                        description:
                            - Enable/disable Distributed Automatic Radio Resource Provisioning (DARRP) to make sure the radio is always using the most optimal
                               channel (default = disable).
                        choices:
                            - enable
                            - disable
                    dtim:
                        description:
                            - DTIM interval. The frequency to transmit Delivery Traffic Indication Message (or Map) (DTIM) messages (1 - 255, default = 1).
                               Set higher to save client battery life.
                    frag-threshold:
                        description:
                            - Maximum packet size that can be sent without fragmentation (800 - 2346 bytes, default = 2346).
                    frequency-handoff:
                        description:
                            - Enable/disable frequency handoff of clients to other channels (default = disable).
                        choices:
                            - enable
                            - disable
                    max-clients:
                        description:
                            - Maximum number of stations (STAs) or WiFi clients supported by the radio. Range depends on the hardware.
                    max-distance:
                        description:
                            - Maximum expected distance between the AP and clients (0 - 54000 m, default = 0).
                    mode:
                        description:
                            - Mode of radio 1. Radio 1 can be disabled, configured as an access point, a rogue AP monitor, or a sniffer.
                        choices:
                            - disabled
                            - ap
                            - monitor
                            - sniffer
                    power-level:
                        description:
                            - Radio power level as a percentage of the maximum transmit power (0 - 100, default = 100).
                    powersave-optimize:
                        description:
                            - Enable client power-saving features such as TIM, AC VO, and OBSS etc.
                        choices:
                            - tim
                            - ac-vo
                            - no-obss-scan
                            - no-11b-rate
                            - client-rate-follow
                    protection-mode:
                        description:
                            - Enable/disable 802.11g protection modes to support backwards compatibility with older clients (rtscts, ctsonly, disable).
                        choices:
                            - rtscts
                            - ctsonly
                            - disable
                    radio-id:
                        description:
                            - radio-id
                    rts-threshold:
                        description:
                            - Maximum packet size for RTS transmissions, specifying the maximum size of a data packet before RTS/CTS (256 - 2346 bytes,
                               default = 2346).
                    short-guard-interval:
                        description:
                            - Use either the short guard interval (Short GI) of 400 ns or the long guard interval (Long GI) of 800 ns.
                        choices:
                            - enable
                            - disable
                    spectrum-analysis:
                        description:
                            - Enable/disable spectrum analysis to find interference that would negatively impact wireless performance.
                        choices:
                            - enable
                            - disable
                    transmit-optimize:
                        description:
                            - Packet transmission optimization options including power saving, aggregation limiting, retry limiting, etc. All are enabled by
                               default.
                        choices:
                            - disable
                            - power-save
                            - aggr-limit
                            - retry-limit
                            - send-bar
                    vap-all:
                        description:
                            - Enable/disable the automatic inheritance of all Virtual Access Points (VAPs) (default = enable).
                        choices:
                            - enable
                            - disable
                    vaps:
                        description:
                            - Manually selected list of Virtual Access Points (VAPs).
                        suboptions:
                            name:
                                description:
                                    - Virtual Access Point (VAP) name. Source wireless-controller.vap-group.name wireless-controller.vap.name.
                                required: true
                    wids-profile:
                        description:
                            - Wireless Intrusion Detection System (WIDS) profile name to assign to the radio. Source wireless-controller.wids-profile.name.
            radio-2:
                description:
                    - Configuration options for radio 2.
                suboptions:
                    amsdu:
                        description:
                            - Enable/disable 802.11n AMSDU support. AMSDU can improve performance if supported by your WiFi clients (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-handoff:
                        description:
                            - Enable/disable AP handoff of clients to other APs (default = disable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-addr:
                        description:
                            - MAC address to monitor.
                    ap-sniffer-bufsize:
                        description:
                            - Sniffer buffer size (1 - 32 MB, default = 16).
                    ap-sniffer-chan:
                        description:
                            - Channel on which to operate the sniffer (default = 6).
                    ap-sniffer-ctl:
                        description:
                            - Enable/disable sniffer on WiFi control frame (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-data:
                        description:
                            - Enable/disable sniffer on WiFi data frame (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-mgmt-beacon:
                        description:
                            - Enable/disable sniffer on WiFi management Beacon frames (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-mgmt-other:
                        description:
                            - Enable/disable sniffer on WiFi management other frames  (default = enable).
                        choices:
                            - enable
                            - disable
                    ap-sniffer-mgmt-probe:
                        description:
                            - Enable/disable sniffer on WiFi management probe frames (default = enable).
                        choices:
                            - enable
                            - disable
                    auto-power-high:
                        description:
                            - Automatic transmit power high limit in dBm (the actual range of transmit power depends on the AP platform type).
                    auto-power-level:
                        description:
                            - Enable/disable automatic power-level adjustment to prevent co-channel interference (default = disable).
                        choices:
                            - enable
                            - disable
                    auto-power-low:
                        description:
                            - Automatic transmission power low limit in dBm (the actual range of transmit power depends on the AP platform type).
                    band:
                        description:
                            - WiFi band that Radio 2 operates on.
                        choices:
                            - 802.11a
                            - 802.11b
                            - 802.11g
                            - 802.11n
                            - 802.11n-5G
                            - 802.11ac
                            - 802.11n,g-only
                            - 802.11g-only
                            - 802.11n-only
                            - 802.11n-5G-only
                            - 802.11ac,n-only
                            - 802.11ac-only
                    bandwidth-admission-control:
                        description:
                            - Enable/disable WiFi multimedia (WMM) bandwidth admission control to optimize WiFi bandwidth use. A request to join the wireless
                               network is only allowed if the access point has enough bandwidth to support it.
                        choices:
                            - enable
                            - disable
                    bandwidth-capacity:
                        description:
                            - Maximum bandwidth capacity allowed (1 - 600000 Kbps, default = 2000).
                    beacon-interval:
                        description:
                            - Beacon interval. The time between beacon frames in msec (the actual range of beacon interval depends on the AP platform type,
                               default = 100).
                    call-admission-control:
                        description:
                            - Enable/disable WiFi multimedia (WMM) call admission control to optimize WiFi bandwidth use for VoIP calls. New VoIP calls are
                               only accepted if there is enough bandwidth available to support them.
                        choices:
                            - enable
                            - disable
                    call-capacity:
                        description:
                            - Maximum number of Voice over WLAN (VoWLAN) phones supported by the radio (0 - 60, default = 10).
                    channel:
                        description:
                            - Selected list of wireless radio channels.
                        suboptions:
                            chan:
                                description:
                                    - Channel number.
                                required: true
                    channel-bonding:
                        description:
                            - "Channel bandwidth: 80, 40, or 20MHz. Channels may use both 20 and 40 by enabling coexistence."
                        choices:
                            - 80MHz
                            - 40MHz
                            - 20MHz
                    channel-utilization:
                        description:
                            - Enable/disable measuring channel utilization.
                        choices:
                            - enable
                            - disable
                    coexistence:
                        description:
                            - Enable/disable allowing both HT20 and HT40 on the same radio (default = enable).
                        choices:
                            - enable
                            - disable
                    darrp:
                        description:
                            - Enable/disable Distributed Automatic Radio Resource Provisioning (DARRP) to make sure the radio is always using the most optimal
                               channel (default = disable).
                        choices:
                            - enable
                            - disable
                    dtim:
                        description:
                            - DTIM interval. The frequency to transmit Delivery Traffic Indication Message (or Map) (DTIM) messages (1 - 255, default = 1).
                               Set higher to save client battery life.
                    frag-threshold:
                        description:
                            - Maximum packet size that can be sent without fragmentation (800 - 2346 bytes, default = 2346).
                    frequency-handoff:
                        description:
                            - Enable/disable frequency handoff of clients to other channels (default = disable).
                        choices:
                            - enable
                            - disable
                    max-clients:
                        description:
                            - Maximum number of stations (STAs) or WiFi clients supported by the radio. Range depends on the hardware.
                    max-distance:
                        description:
                            - Maximum expected distance between the AP and clients (0 - 54000 m, default = 0).
                    mode:
                        description:
                            - Mode of radio 2. Radio 2 can be disabled, configured as an access point, a rogue AP monitor, or a sniffer.
                        choices:
                            - disabled
                            - ap
                            - monitor
                            - sniffer
                    power-level:
                        description:
                            - Radio power level as a percentage of the maximum transmit power (0 - 100, default = 100).
                    powersave-optimize:
                        description:
                            - Enable client power-saving features such as TIM, AC VO, and OBSS etc.
                        choices:
                            - tim
                            - ac-vo
                            - no-obss-scan
                            - no-11b-rate
                            - client-rate-follow
                    protection-mode:
                        description:
                            - Enable/disable 802.11g protection modes to support backwards compatibility with older clients (rtscts, ctsonly, disable).
                        choices:
                            - rtscts
                            - ctsonly
                            - disable
                    radio-id:
                        description:
                            - radio-id
                    rts-threshold:
                        description:
                            - Maximum packet size for RTS transmissions, specifying the maximum size of a data packet before RTS/CTS (256 - 2346 bytes,
                               default = 2346).
                    short-guard-interval:
                        description:
                            - Use either the short guard interval (Short GI) of 400 ns or the long guard interval (Long GI) of 800 ns.
                        choices:
                            - enable
                            - disable
                    spectrum-analysis:
                        description:
                            - Enable/disable spectrum analysis to find interference that would negatively impact wireless performance.
                        choices:
                            - enable
                            - disable
                    transmit-optimize:
                        description:
                            - Packet transmission optimization options including power saving, aggregation limiting, retry limiting, etc. All are enabled by
                               default.
                        choices:
                            - disable
                            - power-save
                            - aggr-limit
                            - retry-limit
                            - send-bar
                    vap-all:
                        description:
                            - Enable/disable the automatic inheritance of all Virtual Access Points (VAPs) (default = enable).
                        choices:
                            - enable
                            - disable
                    vaps:
                        description:
                            - Manually selected list of Virtual Access Points (VAPs).
                        suboptions:
                            name:
                                description:
                                    - Virtual Access Point (VAP) name. Source wireless-controller.vap-group.name wireless-controller.vap.name.
                                required: true
                    wids-profile:
                        description:
                            - Wireless Intrusion Detection System (WIDS) profile name to assign to the radio. Source wireless-controller.wids-profile.name.
            split-tunneling-acl:
                description:
                    - Split tunneling ACL filter list.
                suboptions:
                    dest-ip:
                        description:
                            - Destination IP and mask for the split-tunneling subnet.
                    id:
                        description:
                            - ID.
                        required: true
            split-tunneling-acl-local-ap-subnet:
                description:
                    - Enable/disable automatically adding local subnetwork of FortiAP to split-tunneling ACL (default = disable).
                choices:
                    - enable
                    - disable
            split-tunneling-acl-path:
                description:
                    - Split tunneling ACL path is local/tunnel.
                choices:
                    - tunnel
                    - local
            tun-mtu-downlink:
                description:
                    - Downlink CAPWAP tunnel MTU (0, 576, or 1500 bytes, default = 0).
            tun-mtu-uplink:
                description:
                    - Uplink CAPWAP tunnel MTU (0, 576, or 1500 bytes, default = 0).
            wan-port-mode:
                description:
                    - Enable/disable using a WAN port as a LAN port.
                choices:
                    - wan-lan
                    - wan-only
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure WTP profiles or FortiAP profiles that define radio settings for manageable FortiAP platforms.
    fortios_wireless_controller_wtp_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      wireless_controller_wtp_profile:
        state: "present"
        allowaccess: "telnet"
        ap-country: "NA"
        ble-profile: "<your_own_value> (source wireless-controller.ble-profile.name)"
        comment: "Comment."
        control-message-offload: "ebp-frame"
        deny-mac-list:
         -
            id:  "9"
            mac: "<your_own_value>"
        dtls-in-kernel: "enable"
        dtls-policy: "clear-text"
        energy-efficient-ethernet: "enable"
        ext-info-enable: "enable"
        handoff-roaming: "enable"
        handoff-rssi: "16"
        handoff-sta-thresh: "17"
        ip-fragment-preventing: "tcp-mss-adjust"
        lan:
            port-mode: "offline"
            port-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port1-mode: "offline"
            port1-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port2-mode: "offline"
            port2-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port3-mode: "offline"
            port3-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port4-mode: "offline"
            port4-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port5-mode: "offline"
            port5-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port6-mode: "offline"
            port6-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port7-mode: "offline"
            port7-ssid: "<your_own_value> (source wireless-controller.vap.name)"
            port8-mode: "offline"
            port8-ssid: "<your_own_value> (source wireless-controller.vap.name)"
        lbs:
            aeroscout: "enable"
            aeroscout-ap-mac: "bssid"
            aeroscout-mmu-report: "enable"
            aeroscout-mu: "enable"
            aeroscout-mu-factor: "43"
            aeroscout-mu-timeout: "44"
            aeroscout-server-ip: "<your_own_value>"
            aeroscout-server-port: "46"
            ekahau-blink-mode: "enable"
            ekahau-tag: "<your_own_value>"
            erc-server-ip: "<your_own_value>"
            erc-server-port: "50"
            fortipresence: "foreign"
            fortipresence-frequency: "52"
            fortipresence-port: "53"
            fortipresence-project: "<your_own_value>"
            fortipresence-rogue: "enable"
            fortipresence-secret: "<your_own_value>"
            fortipresence-server: "<your_own_value>"
            fortipresence-unassoc: "enable"
            station-locate: "enable"
        led-schedules:
         -
            name: "default_name_61 (source firewall.schedule.group.name firewall.schedule.recurring.name)"
        led-state: "enable"
        lldp: "enable"
        login-passwd: "<your_own_value>"
        login-passwd-change: "yes"
        max-clients: "66"
        name: "default_name_67"
        platform:
            type: "AP-11N"
        poe-mode: "auto"
        radio-1:
            amsdu: "enable"
            ap-handoff: "enable"
            ap-sniffer-addr: "<your_own_value>"
            ap-sniffer-bufsize: "75"
            ap-sniffer-chan: "76"
            ap-sniffer-ctl: "enable"
            ap-sniffer-data: "enable"
            ap-sniffer-mgmt-beacon: "enable"
            ap-sniffer-mgmt-other: "enable"
            ap-sniffer-mgmt-probe: "enable"
            auto-power-high: "82"
            auto-power-level: "enable"
            auto-power-low: "84"
            band: "802.11a"
            bandwidth-admission-control: "enable"
            bandwidth-capacity: "87"
            beacon-interval: "88"
            call-admission-control: "enable"
            call-capacity: "90"
            channel:
             -
                chan: "<your_own_value>"
            channel-bonding: "80MHz"
            channel-utilization: "enable"
            coexistence: "enable"
            darrp: "enable"
            dtim: "97"
            frag-threshold: "98"
            frequency-handoff: "enable"
            max-clients: "100"
            max-distance: "101"
            mode: "disabled"
            power-level: "103"
            powersave-optimize: "tim"
            protection-mode: "rtscts"
            radio-id: "106"
            rts-threshold: "107"
            short-guard-interval: "enable"
            spectrum-analysis: "enable"
            transmit-optimize: "disable"
            vap-all: "enable"
            vaps:
             -
                name: "default_name_113 (source wireless-controller.vap-group.name wireless-controller.vap.name)"
            wids-profile: "<your_own_value> (source wireless-controller.wids-profile.name)"
        radio-2:
            amsdu: "enable"
            ap-handoff: "enable"
            ap-sniffer-addr: "<your_own_value>"
            ap-sniffer-bufsize: "119"
            ap-sniffer-chan: "120"
            ap-sniffer-ctl: "enable"
            ap-sniffer-data: "enable"
            ap-sniffer-mgmt-beacon: "enable"
            ap-sniffer-mgmt-other: "enable"
            ap-sniffer-mgmt-probe: "enable"
            auto-power-high: "126"
            auto-power-level: "enable"
            auto-power-low: "128"
            band: "802.11a"
            bandwidth-admission-control: "enable"
            bandwidth-capacity: "131"
            beacon-interval: "132"
            call-admission-control: "enable"
            call-capacity: "134"
            channel:
             -
                chan: "<your_own_value>"
            channel-bonding: "80MHz"
            channel-utilization: "enable"
            coexistence: "enable"
            darrp: "enable"
            dtim: "141"
            frag-threshold: "142"
            frequency-handoff: "enable"
            max-clients: "144"
            max-distance: "145"
            mode: "disabled"
            power-level: "147"
            powersave-optimize: "tim"
            protection-mode: "rtscts"
            radio-id: "150"
            rts-threshold: "151"
            short-guard-interval: "enable"
            spectrum-analysis: "enable"
            transmit-optimize: "disable"
            vap-all: "enable"
            vaps:
             -
                name: "default_name_157 (source wireless-controller.vap-group.name wireless-controller.vap.name)"
            wids-profile: "<your_own_value> (source wireless-controller.wids-profile.name)"
        split-tunneling-acl:
         -
            dest-ip: "<your_own_value>"
            id:  "161"
        split-tunneling-acl-local-ap-subnet: "enable"
        split-tunneling-acl-path: "tunnel"
        tun-mtu-downlink: "164"
        tun-mtu-uplink: "165"
        wan-port-mode: "wan-lan"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_wireless_controller_wtp_profile_data(json):
    option_list = ['allowaccess', 'ap-country', 'ble-profile',
                   'comment', 'control-message-offload', 'deny-mac-list',
                   'dtls-in-kernel', 'dtls-policy', 'energy-efficient-ethernet',
                   'ext-info-enable', 'handoff-roaming', 'handoff-rssi',
                   'handoff-sta-thresh', 'ip-fragment-preventing', 'lan',
                   'lbs', 'led-schedules', 'led-state',
                   'lldp', 'login-passwd', 'login-passwd-change',
                   'max-clients', 'name', 'platform',
                   'poe-mode', 'radio-1', 'radio-2',
                   'split-tunneling-acl', 'split-tunneling-acl-local-ap-subnet', 'split-tunneling-acl-path',
                   'tun-mtu-downlink', 'tun-mtu-uplink', 'wan-port-mode']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def wireless_controller_wtp_profile(data, fos):
    vdom = data['vdom']
    wireless_controller_wtp_profile_data = data['wireless_controller_wtp_profile']
    filtered_data = filter_wireless_controller_wtp_profile_data(wireless_controller_wtp_profile_data)

    if wireless_controller_wtp_profile_data['state'] == "present":
        return fos.set('wireless-controller',
                       'wtp-profile',
                       data=filtered_data,
                       vdom=vdom)

    elif wireless_controller_wtp_profile_data['state'] == "absent":
        return fos.delete('wireless-controller',
                          'wtp-profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_wireless_controller(data, fos):
    login(data, fos)

    if data['wireless_controller_wtp_profile']:
        resp = wireless_controller_wtp_profile(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "wireless_controller_wtp_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "allowaccess": {"required": False, "type": "str",
                                "choices": ["telnet", "http", "https",
                                            "ssh"]},
                "ap-country": {"required": False, "type": "str",
                               "choices": ["NA", "AL", "DZ",
                                           "AO", "AR", "AM",
                                           "AU", "AT", "AZ",
                                           "BH", "BD", "BB",
                                           "BY", "BE", "BZ",
                                           "BO", "BA", "BR",
                                           "BN", "BG", "KH",
                                           "CL", "CN", "CO",
                                           "CR", "HR", "CY",
                                           "CZ", "DK", "DO",
                                           "EC", "EG", "SV",
                                           "EE", "FI", "FR",
                                           "GE", "DE", "GR",
                                           "GL", "GD", "GU",
                                           "GT", "HT", "HN",
                                           "HK", "HU", "IS",
                                           "IN", "ID", "IR",
                                           "IE", "IL", "IT",
                                           "JM", "JO", "KZ",
                                           "KE", "KP", "KR",
                                           "KW", "LV", "LB",
                                           "LI", "LT", "LU",
                                           "MO", "MK", "MY",
                                           "MT", "MX", "MC",
                                           "MA", "MZ", "MM",
                                           "NP", "NL", "AN",
                                           "AW", "NZ", "NO",
                                           "OM", "PK", "PA",
                                           "PG", "PY", "PE",
                                           "PH", "PL", "PT",
                                           "PR", "QA", "RO",
                                           "RU", "RW", "SA",
                                           "RS", "ME", "SG",
                                           "SK", "SI", "ZA",
                                           "ES", "LK", "SE",
                                           "SD", "CH", "SY",
                                           "TW", "TZ", "TH",
                                           "TT", "TN", "TR",
                                           "AE", "UA", "GB",
                                           "US", "PS", "UY",
                                           "UZ", "VE", "VN",
                                           "YE", "ZB", "ZW",
                                           "JP", "CA"]},
                "ble-profile": {"required": False, "type": "str"},
                "comment": {"required": False, "type": "str"},
                "control-message-offload": {"required": False, "type": "str",
                                            "choices": ["ebp-frame", "aeroscout-tag", "ap-list",
                                                        "sta-list", "sta-cap-list", "stats",
                                                        "aeroscout-mu"]},
                "deny-mac-list": {"required": False, "type": "list",
                                  "options": {
                                      "id": {"required": True, "type": "int"},
                                      "mac": {"required": False, "type": "str"}
                                  }},
                "dtls-in-kernel": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "dtls-policy": {"required": False, "type": "str",
                                "choices": ["clear-text", "dtls-enabled", "ipsec-vpn"]},
                "energy-efficient-ethernet": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "ext-info-enable": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "handoff-roaming": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "handoff-rssi": {"required": False, "type": "int"},
                "handoff-sta-thresh": {"required": False, "type": "int"},
                "ip-fragment-preventing": {"required": False, "type": "str",
                                           "choices": ["tcp-mss-adjust", "icmp-unreachable"]},
                "lan": {"required": False, "type": "dict",
                        "options": {
                            "port-mode": {"required": False, "type": "str",
                                          "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                      "bridge-to-ssid"]},
                            "port-ssid": {"required": False, "type": "str"},
                            "port1-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port1-ssid": {"required": False, "type": "str"},
                            "port2-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port2-ssid": {"required": False, "type": "str"},
                            "port3-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port3-ssid": {"required": False, "type": "str"},
                            "port4-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port4-ssid": {"required": False, "type": "str"},
                            "port5-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port5-ssid": {"required": False, "type": "str"},
                            "port6-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port6-ssid": {"required": False, "type": "str"},
                            "port7-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port7-ssid": {"required": False, "type": "str"},
                            "port8-mode": {"required": False, "type": "str",
                                           "choices": ["offline", "nat-to-wan", "bridge-to-wan",
                                                       "bridge-to-ssid"]},
                            "port8-ssid": {"required": False, "type": "str"}
                        }},
                "lbs": {"required": False, "type": "dict",
                        "options": {
                            "aeroscout": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                            "aeroscout-ap-mac": {"required": False, "type": "str",
                                                 "choices": ["bssid", "board-mac"]},
                            "aeroscout-mmu-report": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                            "aeroscout-mu": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                            "aeroscout-mu-factor": {"required": False, "type": "int"},
                            "aeroscout-mu-timeout": {"required": False, "type": "int"},
                            "aeroscout-server-ip": {"required": False, "type": "str"},
                            "aeroscout-server-port": {"required": False, "type": "int"},
                            "ekahau-blink-mode": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                            "ekahau-tag": {"required": False, "type": "str"},
                            "erc-server-ip": {"required": False, "type": "str"},
                            "erc-server-port": {"required": False, "type": "int"},
                            "fortipresence": {"required": False, "type": "str",
                                              "choices": ["foreign", "both", "disable"]},
                            "fortipresence-frequency": {"required": False, "type": "int"},
                            "fortipresence-port": {"required": False, "type": "int"},
                            "fortipresence-project": {"required": False, "type": "str"},
                            "fortipresence-rogue": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                            "fortipresence-secret": {"required": False, "type": "str"},
                            "fortipresence-server": {"required": False, "type": "str"},
                            "fortipresence-unassoc": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                            "station-locate": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]}
                        }},
                "led-schedules": {"required": False, "type": "list",
                                  "options": {
                                      "name": {"required": True, "type": "str"}
                                  }},
                "led-state": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "lldp": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "login-passwd": {"required": False, "type": "str"},
                "login-passwd-change": {"required": False, "type": "str",
                                        "choices": ["yes", "default", "no"]},
                "max-clients": {"required": False, "type": "int"},
                "name": {"required": True, "type": "str"},
                "platform": {"required": False, "type": "dict",
                             "options": {
                                 "type": {"required": False, "type": "str",
                                          "choices": ["AP-11N", "220B", "210B",
                                                      "222B", "112B", "320B",
                                                      "11C", "14C", "223B",
                                                      "28C", "320C", "221C",
                                                      "25D", "222C", "224D",
                                                      "214B", "21D", "24D",
                                                      "112D", "223C", "321C",
                                                      "C220C", "C225C", "C23JD",
                                                      "C24JE", "S321C", "S322C",
                                                      "S323C", "S311C", "S313C",
                                                      "S321CR", "S322CR", "S323CR",
                                                      "S421E", "S422E", "S423E",
                                                      "421E", "423E", "221E",
                                                      "222E", "223E", "224E",
                                                      "S221E", "S223E", "U421E",
                                                      "U422EV", "U423E", "U221EV",
                                                      "U223EV", "U24JEV", "U321EV",
                                                      "U323EV"]}
                             }},
                "poe-mode": {"required": False, "type": "str",
                             "choices": ["auto", "8023af", "8023at",
                                         "power-adapter"]},
                "radio-1": {"required": False, "type": "dict",
                            "options": {
                                "amsdu": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                                "ap-handoff": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                                "ap-sniffer-addr": {"required": False, "type": "str"},
                                "ap-sniffer-bufsize": {"required": False, "type": "int"},
                                "ap-sniffer-chan": {"required": False, "type": "int"},
                                "ap-sniffer-ctl": {"required": False, "type": "str",
                                                   "choices": ["enable", "disable"]},
                                "ap-sniffer-data": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                                "ap-sniffer-mgmt-beacon": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]},
                                "ap-sniffer-mgmt-other": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]},
                                "ap-sniffer-mgmt-probe": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]},
                                "auto-power-high": {"required": False, "type": "int"},
                                "auto-power-level": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                "auto-power-low": {"required": False, "type": "int"},
                                "band": {"required": False, "type": "str",
                                         "choices": ["802.11a", "802.11b", "802.11g",
                                                     "802.11n", "802.11n-5G", "802.11ac",
                                                     "802.11n,g-only", "802.11g-only", "802.11n-only",
                                                     "802.11n-5G-only", "802.11ac,n-only", "802.11ac-only"]},
                                "bandwidth-admission-control": {"required": False, "type": "str",
                                                                "choices": ["enable", "disable"]},
                                "bandwidth-capacity": {"required": False, "type": "int"},
                                "beacon-interval": {"required": False, "type": "int"},
                                "call-admission-control": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]},
                                "call-capacity": {"required": False, "type": "int"},
                                "channel": {"required": False, "type": "list",
                                            "options": {
                                                "chan": {"required": True, "type": "str"}
                                            }},
                                "channel-bonding": {"required": False, "type": "str",
                                                    "choices": ["80MHz", "40MHz", "20MHz"]},
                                "channel-utilization": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                "coexistence": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                                "darrp": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                                "dtim": {"required": False, "type": "int"},
                                "frag-threshold": {"required": False, "type": "int"},
                                "frequency-handoff": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                "max-clients": {"required": False, "type": "int"},
                                "max-distance": {"required": False, "type": "int"},
                                "mode": {"required": False, "type": "str",
                                         "choices": ["disabled", "ap", "monitor",
                                                     "sniffer"]},
                                "power-level": {"required": False, "type": "int"},
                                "powersave-optimize": {"required": False, "type": "str",
                                                       "choices": ["tim", "ac-vo", "no-obss-scan",
                                                                   "no-11b-rate", "client-rate-follow"]},
                                "protection-mode": {"required": False, "type": "str",
                                                    "choices": ["rtscts", "ctsonly", "disable"]},
                                "radio-id": {"required": False, "type": "int"},
                                "rts-threshold": {"required": False, "type": "int"},
                                "short-guard-interval": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]},
                                "spectrum-analysis": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                "transmit-optimize": {"required": False, "type": "str",
                                                      "choices": ["disable", "power-save", "aggr-limit",
                                                                  "retry-limit", "send-bar"]},
                                "vap-all": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                                "vaps": {"required": False, "type": "list",
                                         "options": {
                                             "name": {"required": True, "type": "str"}
                                         }},
                                "wids-profile": {"required": False, "type": "str"}
                            }},
                "radio-2": {"required": False, "type": "dict",
                            "options": {
                                "amsdu": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                                "ap-handoff": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                                "ap-sniffer-addr": {"required": False, "type": "str"},
                                "ap-sniffer-bufsize": {"required": False, "type": "int"},
                                "ap-sniffer-chan": {"required": False, "type": "int"},
                                "ap-sniffer-ctl": {"required": False, "type": "str",
                                                   "choices": ["enable", "disable"]},
                                "ap-sniffer-data": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                                "ap-sniffer-mgmt-beacon": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]},
                                "ap-sniffer-mgmt-other": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]},
                                "ap-sniffer-mgmt-probe": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]},
                                "auto-power-high": {"required": False, "type": "int"},
                                "auto-power-level": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                "auto-power-low": {"required": False, "type": "int"},
                                "band": {"required": False, "type": "str",
                                         "choices": ["802.11a", "802.11b", "802.11g",
                                                     "802.11n", "802.11n-5G", "802.11ac",
                                                     "802.11n,g-only", "802.11g-only", "802.11n-only",
                                                     "802.11n-5G-only", "802.11ac,n-only", "802.11ac-only"]},
                                "bandwidth-admission-control": {"required": False, "type": "str",
                                                                "choices": ["enable", "disable"]},
                                "bandwidth-capacity": {"required": False, "type": "int"},
                                "beacon-interval": {"required": False, "type": "int"},
                                "call-admission-control": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]},
                                "call-capacity": {"required": False, "type": "int"},
                                "channel": {"required": False, "type": "list",
                                            "options": {
                                                "chan": {"required": True, "type": "str"}
                                            }},
                                "channel-bonding": {"required": False, "type": "str",
                                                    "choices": ["80MHz", "40MHz", "20MHz"]},
                                "channel-utilization": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                "coexistence": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                                "darrp": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                                "dtim": {"required": False, "type": "int"},
                                "frag-threshold": {"required": False, "type": "int"},
                                "frequency-handoff": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                "max-clients": {"required": False, "type": "int"},
                                "max-distance": {"required": False, "type": "int"},
                                "mode": {"required": False, "type": "str",
                                         "choices": ["disabled", "ap", "monitor",
                                                     "sniffer"]},
                                "power-level": {"required": False, "type": "int"},
                                "powersave-optimize": {"required": False, "type": "str",
                                                       "choices": ["tim", "ac-vo", "no-obss-scan",
                                                                   "no-11b-rate", "client-rate-follow"]},
                                "protection-mode": {"required": False, "type": "str",
                                                    "choices": ["rtscts", "ctsonly", "disable"]},
                                "radio-id": {"required": False, "type": "int"},
                                "rts-threshold": {"required": False, "type": "int"},
                                "short-guard-interval": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]},
                                "spectrum-analysis": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                "transmit-optimize": {"required": False, "type": "str",
                                                      "choices": ["disable", "power-save", "aggr-limit",
                                                                  "retry-limit", "send-bar"]},
                                "vap-all": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                                "vaps": {"required": False, "type": "list",
                                         "options": {
                                             "name": {"required": True, "type": "str"}
                                         }},
                                "wids-profile": {"required": False, "type": "str"}
                            }},
                "split-tunneling-acl": {"required": False, "type": "list",
                                        "options": {
                                            "dest-ip": {"required": False, "type": "str"},
                                            "id": {"required": True, "type": "int"}
                                        }},
                "split-tunneling-acl-local-ap-subnet": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                "split-tunneling-acl-path": {"required": False, "type": "str",
                                             "choices": ["tunnel", "local"]},
                "tun-mtu-downlink": {"required": False, "type": "int"},
                "tun-mtu-uplink": {"required": False, "type": "int"},
                "wan-port-mode": {"required": False, "type": "str",
                                  "choices": ["wan-lan", "wan-only"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_wireless_controller(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test for ONTAP Command Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_command \
    import NetAppONTAPCommand as my_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, parm1=None):
        ''' save arguments '''
        self.type = kind
        self.parm1 = parm1
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        # print(xml.to_string())

        if self.type == 'version':
            priv = xml.get_child_content('priv')
            xml = self.build_version(priv)
        self.xml_out = xml
        return xml

    @staticmethod
    def build_version(priv):
        ''' build xml data for version '''
        prefix = 'NetApp Release'
        if priv == 'advanced':
            prefix = '\n' + prefix
        xml = netapp_utils.zapi.NaElement('results')
        xml.add_new_child('cli-output', prefix)
        # print(xml.to_string())
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection(kind='version')
        # whether to use a mock or a simulator
        self.use_vsim = False

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            my_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    @staticmethod
    def set_default_args(vsim=False):
        ''' populate hostname/username/password '''
        if vsim:
            hostname = '10.193.78.219'
            username = 'admin'
            password = 'netapp1!'
        else:
            hostname = 'hostname'
            username = 'username'
            password = 'password'
        return dict({
            'hostname': hostname,
            'username': username,
            'password': password,
            'https': True,
            'validate_certs': False
        })

    def call_command(self, module_args, vsim=False):
        ''' utility function to call apply '''
        module_args.update(self.set_default_args(vsim=vsim))
        set_module_args(module_args)
        my_obj = my_module()
        my_obj.asup_log_for_cserver = Mock(return_value=None)
        if not vsim:
            # mock the connection
            my_obj.server = self.server
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        msg = exc.value.args[0]['msg']
        return msg

    def test_default_priv(self):
        ''' make sure privilege is not required '''
        module_args = {
            'command': 'version',
        }
        msg = self.call_command(module_args, vsim=self.use_vsim)
        needle = b'<cli-output>NetApp Release'
        assert needle in msg
        print('Version (raw): %s' % msg)

    def test_admin_priv(self):
        ''' make sure admin is accepted '''
        module_args = {
            'command': 'version',
            'privilege': 'admin',
        }
        msg = self.call_command(module_args, vsim=self.use_vsim)
        needle = b'<cli-output>NetApp Release'
        assert needle in msg
        print('Version (raw): %s' % msg)

    def test_advanced_priv(self):
        ''' make sure advanced is not required '''
        module_args = {
            'command': 'version',
            'privilege': 'advanced',
        }
        msg = self.call_command(module_args, vsim=self.use_vsim)
        # Interestingly, the ZAPI returns a slightly different response
        needle = b'<cli-output>\nNetApp Release'
        assert needle in msg
        print('Version (raw): %s' % msg)



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_firewall_vip
short_description: Configure virtual IP for IPv4 in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure firewall feature and vip category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    firewall_vip:
        description:
            - Configure virtual IP for IPv4.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            arp-reply:
                description:
                    - Enable to respond to ARP requests for this virtual IP address. Enabled by default.
                choices:
                    - disable
                    - enable
            color:
                description:
                    - Color of icon on the GUI.
            comment:
                description:
                    - Comment.
            dns-mapping-ttl:
                description:
                    - DNS mapping TTL (Set to zero to use TTL in DNS response, default = 0).
            extaddr:
                description:
                    - External FQDN address name.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address.name firewall.addrgrp.name.
                        required: true
            extintf:
                description:
                    - Interface connected to the source network that receives the packets that will be forwarded to the destination network. Source system
                      .interface.name.
            extip:
                description:
                    - IP address or address range on the external interface that you want to map to an address or address range on the destination network.
            extport:
                description:
                    - Incoming port number range that you want to map to a port number range on the destination network.
            gratuitous-arp-interval:
                description:
                    - Enable to have the VIP send gratuitous ARPs. 0=disabled. Set from 5 up to 8640000 seconds to enable.
            http-cookie-age:
                description:
                    - Time in minutes that client web browsers should keep a cookie. Default is 60 seconds. 0 = no time limit.
            http-cookie-domain:
                description:
                    - Domain that HTTP cookie persistence should apply to.
            http-cookie-domain-from-host:
                description:
                    - Enable/disable use of HTTP cookie domain from host field in HTTP.
                choices:
                    - disable
                    - enable
            http-cookie-generation:
                description:
                    - Generation of HTTP cookie to be accepted. Changing invalidates all existing cookies.
            http-cookie-path:
                description:
                    - Limit HTTP cookie persistence to the specified path.
            http-cookie-share:
                description:
                    - Control sharing of cookies across virtual servers. same-ip means a cookie from one virtual server can be used by another. Disable stops
                       cookie sharing.
                choices:
                    - disable
                    - same-ip
            http-ip-header:
                description:
                    - For HTTP multiplexing, enable to add the original client IP address in the XForwarded-For HTTP header.
                choices:
                    - enable
                    - disable
            http-ip-header-name:
                description:
                    - For HTTP multiplexing, enter a custom HTTPS header name. The original client IP address is added to this header. If empty,
                       X-Forwarded-For is used.
            http-multiplex:
                description:
                    - Enable/disable HTTP multiplexing.
                choices:
                    - enable
                    - disable
            https-cookie-secure:
                description:
                    - Enable/disable verification that inserted HTTPS cookies are secure.
                choices:
                    - disable
                    - enable
            id:
                description:
                    - Custom defined ID.
            ldb-method:
                description:
                    - Method used to distribute sessions to real servers.
                choices:
                    - static
                    - round-robin
                    - weighted
                    - least-session
                    - least-rtt
                    - first-alive
                    - http-host
            mapped-addr:
                description:
                    - Mapped FQDN address name. Source firewall.address.name.
            mappedip:
                description:
                    - IP address or address range on the destination network to which the external IP address is mapped.
                suboptions:
                    range:
                        description:
                            - Mapped IP range.
                        required: true
            mappedport:
                description:
                    - Port number range on the destination network to which the external port number range is mapped.
            max-embryonic-connections:
                description:
                    - Maximum number of incomplete connections.
            monitor:
                description:
                    - Name of the health check monitor to use when polling to determine a virtual server's connectivity status.
                suboptions:
                    name:
                        description:
                            - Health monitor name. Source firewall.ldb-monitor.name.
                        required: true
            name:
                description:
                    - Virtual IP name.
                required: true
            nat-source-vip:
                description:
                    - Enable to prevent unintended servers from using a virtual IP. Disable to use the actual IP address of the server as the source address.
                choices:
                    - disable
                    - enable
            outlook-web-access:
                description:
                    - Enable to add the Front-End-Https header for Microsoft Outlook Web Access.
                choices:
                    - disable
                    - enable
            persistence:
                description:
                    - Configure how to make sure that clients connect to the same server every time they make a request that is part of the same session.
                choices:
                    - none
                    - http-cookie
                    - ssl-session-id
            portforward:
                description:
                    - Enable/disable port forwarding.
                choices:
                    - disable
                    - enable
            portmapping-type:
                description:
                    - Port mapping type.
                choices:
                    - 1-to-1
                    - m-to-n
            protocol:
                description:
                    - Protocol to use when forwarding packets.
                choices:
                    - tcp
                    - udp
                    - sctp
                    - icmp
            realservers:
                description:
                    - Select the real servers that this server load balancing VIP will distribute traffic to.
                suboptions:
                    client-ip:
                        description:
                            - Only clients in this IP range can connect to this real server.
                    healthcheck:
                        description:
                            - Enable to check the responsiveness of the real server before forwarding traffic.
                        choices:
                            - disable
                            - enable
                            - vip
                    holddown-interval:
                        description:
                            - Time in seconds that the health check monitor continues to monitor and unresponsive server that should be active.
                    http-host:
                        description:
                            - HTTP server domain name in HTTP header.
                    id:
                        description:
                            - Real server ID.
                        required: true
                    ip:
                        description:
                            - IP address of the real server.
                    max-connections:
                        description:
                            - Max number of active connections that can be directed to the real server. When reached, sessions are sent to other real servers.
                    monitor:
                        description:
                            - Name of the health check monitor to use when polling to determine a virtual server's connectivity status. Source firewall
                              .ldb-monitor.name.
                    port:
                        description:
                            - Port for communicating with the real server. Required if port forwarding is enabled.
                    status:
                        description:
                            - Set the status of the real server to active so that it can accept traffic, or on standby or disabled so no traffic is sent.
                        choices:
                            - active
                            - standby
                            - disable
                    weight:
                        description:
                            - Weight of the real server. If weighted load balancing is enabled, the server with the highest weight gets more connections.
            server-type:
                description:
                    - Protocol to be load balanced by the virtual server (also called the server load balance virtual IP).
                choices:
                    - http
                    - https
                    - imaps
                    - pop3s
                    - smtps
                    - ssl
                    - tcp
                    - udp
                    - ip
            service:
                description:
                    - Service name.
                suboptions:
                    name:
                        description:
                            - Service name. Source firewall.service.custom.name firewall.service.group.name.
                        required: true
            src-filter:
                description:
                    - Source address filter. Each address must be either an IP/subnet (x.x.x.x/n) or a range (x.x.x.x-y.y.y.y). Separate addresses with spaces.
                suboptions:
                    range:
                        description:
                            - Source-filter range.
                        required: true
            srcintf-filter:
                description:
                    - Interfaces to which the VIP applies. Separate the names with spaces.
                suboptions:
                    interface-name:
                        description:
                            - Interface name. Source system.interface.name.
                        required: true
            ssl-algorithm:
                description:
                    - Permitted encryption algorithms for SSL sessions according to encryption strength.
                choices:
                    - high
                    - medium
                    - low
                    - custom
            ssl-certificate:
                description:
                    - The name of the SSL certificate to use for SSL acceleration. Source vpn.certificate.local.name.
            ssl-cipher-suites:
                description:
                    - SSL/TLS cipher suites acceptable from a client, ordered by priority.
                suboptions:
                    cipher:
                        description:
                            - Cipher suite name.
                        choices:
                            - TLS-RSA-WITH-3DES-EDE-CBC-SHA
                            - TLS-DHE-RSA-WITH-DES-CBC-SHA
                            - TLS-DHE-DSS-WITH-DES-CBC-SHA
                    priority:
                        description:
                            - SSL/TLS cipher suites priority.
                        required: true
                    versions:
                        description:
                            - SSL/TLS versions that the cipher suite can be used with.
                        choices:
                            - ssl-3.0
                            - tls-1.0
                            - tls-1.1
                            - tls-1.2
            ssl-client-fallback:
                description:
                    - Enable/disable support for preventing Downgrade Attacks on client connections (RFC 7507).
                choices:
                    - disable
                    - enable
            ssl-client-renegotiation:
                description:
                    - Allow, deny, or require secure renegotiation of client sessions to comply with RFC 5746.
                choices:
                    - allow
                    - deny
                    - secure
            ssl-client-session-state-max:
                description:
                    - Maximum number of client to FortiGate SSL session states to keep.
            ssl-client-session-state-timeout:
                description:
                    - Number of minutes to keep client to FortiGate SSL session state.
            ssl-client-session-state-type:
                description:
                    - How to expire SSL sessions for the segment of the SSL connection between the client and the FortiGate.
                choices:
                    - disable
                    - time
                    - count
                    - both
            ssl-dh-bits:
                description:
                    - Number of bits to use in the Diffie-Hellman exchange for RSA encryption of SSL sessions.
                choices:
                    - 768
                    - 1024
                    - 1536
                    - 2048
                    - 3072
                    - 4096
            ssl-hpkp:
                description:
                    - Enable/disable including HPKP header in response.
                choices:
                    - disable
                    - enable
                    - report-only
            ssl-hpkp-age:
                description:
                    - Number of seconds the client should honour the HPKP setting.
            ssl-hpkp-backup:
                description:
                    - Certificate to generate backup HPKP pin from. Source vpn.certificate.local.name vpn.certificate.ca.name.
            ssl-hpkp-include-subdomains:
                description:
                    - Indicate that HPKP header applies to all subdomains.
                choices:
                    - disable
                    - enable
            ssl-hpkp-primary:
                description:
                    - Certificate to generate primary HPKP pin from. Source vpn.certificate.local.name vpn.certificate.ca.name.
            ssl-hpkp-report-uri:
                description:
                    - URL to report HPKP violations to.
            ssl-hsts:
                description:
                    - Enable/disable including HSTS header in response.
                choices:
                    - disable
                    - enable
            ssl-hsts-age:
                description:
                    - Number of seconds the client should honour the HSTS setting.
            ssl-hsts-include-subdomains:
                description:
                    - Indicate that HSTS header applies to all subdomains.
                choices:
                    - disable
                    - enable
            ssl-http-location-conversion:
                description:
                    - Enable to replace HTTP with HTTPS in the reply's Location HTTP header field.
                choices:
                    - enable
                    - disable
            ssl-http-match-host:
                description:
                    - Enable/disable HTTP host matching for location conversion.
                choices:
                    - enable
                    - disable
            ssl-max-version:
                description:
                    - Highest SSL/TLS version acceptable from a client.
                choices:
                    - ssl-3.0
                    - tls-1.0
                    - tls-1.1
                    - tls-1.2
            ssl-min-version:
                description:
                    - Lowest SSL/TLS version acceptable from a client.
                choices:
                    - ssl-3.0
                    - tls-1.0
                    - tls-1.1
                    - tls-1.2
            ssl-mode:
                description:
                    - Apply SSL offloading between the client and the FortiGate (half) or from the client to the FortiGate and from the FortiGate to the
                       server (full).
                choices:
                    - half
                    - full
            ssl-pfs:
                description:
                    - Select the cipher suites that can be used for SSL perfect forward secrecy (PFS). Applies to both client and server sessions.
                choices:
                    - require
                    - deny
                    - allow
            ssl-send-empty-frags:
                description:
                    - Enable/disable sending empty fragments to avoid CBC IV attacks (SSL 3.0 & TLS 1.0 only). May need to be disabled for compatibility with
                       older systems.
                choices:
                    - enable
                    - disable
            ssl-server-algorithm:
                description:
                    - Permitted encryption algorithms for the server side of SSL full mode sessions according to encryption strength.
                choices:
                    - high
                    - medium
                    - low
                    - custom
                    - client
            ssl-server-cipher-suites:
                description:
                    - SSL/TLS cipher suites to offer to a server, ordered by priority.
                suboptions:
                    cipher:
                        description:
                            - Cipher suite name.
                        choices:
                            - TLS-RSA-WITH-3DES-EDE-CBC-SHA
                            - TLS-DHE-RSA-WITH-DES-CBC-SHA
                            - TLS-DHE-DSS-WITH-DES-CBC-SHA
                    priority:
                        description:
                            - SSL/TLS cipher suites priority.
                        required: true
                    versions:
                        description:
                            - SSL/TLS versions that the cipher suite can be used with.
                        choices:
                            - ssl-3.0
                            - tls-1.0
                            - tls-1.1
                            - tls-1.2
            ssl-server-max-version:
                description:
                    - Highest SSL/TLS version acceptable from a server. Use the client setting by default.
                choices:
                    - ssl-3.0
                    - tls-1.0
                    - tls-1.1
                    - tls-1.2
                    - client
            ssl-server-min-version:
                description:
                    - Lowest SSL/TLS version acceptable from a server. Use the client setting by default.
                choices:
                    - ssl-3.0
                    - tls-1.0
                    - tls-1.1
                    - tls-1.2
                    - client
            ssl-server-session-state-max:
                description:
                    - Maximum number of FortiGate to Server SSL session states to keep.
            ssl-server-session-state-timeout:
                description:
                    - Number of minutes to keep FortiGate to Server SSL session state.
            ssl-server-session-state-type:
                description:
                    - How to expire SSL sessions for the segment of the SSL connection between the server and the FortiGate.
                choices:
                    - disable
                    - time
                    - count
                    - both
            type:
                description:
                    - Configure a static NAT, load balance, server load balance, DNS translation, or FQDN VIP.
                choices:
                    - static-nat
                    - load-balance
                    - server-load-balance
                    - dns-translation
                    - fqdn
            uuid:
                description:
                    - Universally Unique Identifier (UUID; automatically assigned but can be manually reset).
            weblogic-server:
                description:
                    - Enable to add an HTTP header to indicate SSL offloading for a WebLogic server.
                choices:
                    - disable
                    - enable
            websphere-server:
                description:
                    - Enable to add an HTTP header to indicate SSL offloading for a WebSphere server.
                choices:
                    - disable
                    - enable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure virtual IP for IPv4.
    fortios_firewall_vip:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      firewall_vip:
        state: "present"
        arp-reply: "disable"
        color: "4"
        comment: "Comment."
        dns-mapping-ttl: "6"
        extaddr:
         -
            name: "default_name_8 (source firewall.address.name firewall.addrgrp.name)"
        extintf: "<your_own_value> (source system.interface.name)"
        extip: "<your_own_value>"
        extport: "<your_own_value>"
        gratuitous-arp-interval: "12"
        http-cookie-age: "13"
        http-cookie-domain: "<your_own_value>"
        http-cookie-domain-from-host: "disable"
        http-cookie-generation: "16"
        http-cookie-path: "<your_own_value>"
        http-cookie-share: "disable"
        http-ip-header: "enable"
        http-ip-header-name: "<your_own_value>"
        http-multiplex: "enable"
        https-cookie-secure: "disable"
        id:  "23"
        ldb-method: "static"
        mapped-addr: "<your_own_value> (source firewall.address.name)"
        mappedip:
         -
            range: "<your_own_value>"
        mappedport: "<your_own_value>"
        max-embryonic-connections: "29"
        monitor:
         -
            name: "default_name_31 (source firewall.ldb-monitor.name)"
        name: "default_name_32"
        nat-source-vip: "disable"
        outlook-web-access: "disable"
        persistence: "none"
        portforward: "disable"
        portmapping-type: "1-to-1"
        protocol: "tcp"
        realservers:
         -
            client-ip: "<your_own_value>"
            healthcheck: "disable"
            holddown-interval: "42"
            http-host: "myhostname"
            id:  "44"
            ip: "<your_own_value>"
            max-connections: "46"
            monitor: "<your_own_value> (source firewall.ldb-monitor.name)"
            port: "48"
            status: "active"
            weight: "50"
        server-type: "http"
        service:
         -
            name: "default_name_53 (source firewall.service.custom.name firewall.service.group.name)"
        src-filter:
         -
            range: "<your_own_value>"
        srcintf-filter:
         -
            interface-name: "<your_own_value> (source system.interface.name)"
        ssl-algorithm: "high"
        ssl-certificate: "<your_own_value> (source vpn.certificate.local.name)"
        ssl-cipher-suites:
         -
            cipher: "TLS-RSA-WITH-3DES-EDE-CBC-SHA"
            priority: "62"
            versions: "ssl-3.0"
        ssl-client-fallback: "disable"
        ssl-client-renegotiation: "allow"
        ssl-client-session-state-max: "66"
        ssl-client-session-state-timeout: "67"
        ssl-client-session-state-type: "disable"
        ssl-dh-bits: "768"
        ssl-hpkp: "disable"
        ssl-hpkp-age: "71"
        ssl-hpkp-backup: "<your_own_value> (source vpn.certificate.local.name vpn.certificate.ca.name)"
        ssl-hpkp-include-subdomains: "disable"
        ssl-hpkp-primary: "<your_own_value> (source vpn.certificate.local.name vpn.certificate.ca.name)"
        ssl-hpkp-report-uri: "<your_own_value>"
        ssl-hsts: "disable"
        ssl-hsts-age: "77"
        ssl-hsts-include-subdomains: "disable"
        ssl-http-location-conversion: "enable"
        ssl-http-match-host: "enable"
        ssl-max-version: "ssl-3.0"
        ssl-min-version: "ssl-3.0"
        ssl-mode: "half"
        ssl-pfs: "require"
        ssl-send-empty-frags: "enable"
        ssl-server-algorithm: "high"
        ssl-server-cipher-suites:
         -
            cipher: "TLS-RSA-WITH-3DES-EDE-CBC-SHA"
            priority: "89"
            versions: "ssl-3.0"
        ssl-server-max-version: "ssl-3.0"
        ssl-server-min-version: "ssl-3.0"
        ssl-server-session-state-max: "93"
        ssl-server-session-state-timeout: "94"
        ssl-server-session-state-type: "disable"
        type: "static-nat"
        uuid: "<your_own_value>"
        weblogic-server: "disable"
        websphere-server: "disable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_firewall_vip_data(json):
    option_list = ['arp-reply', 'color', 'comment',
                   'dns-mapping-ttl', 'extaddr', 'extintf',
                   'extip', 'extport', 'gratuitous-arp-interval',
                   'http-cookie-age', 'http-cookie-domain', 'http-cookie-domain-from-host',
                   'http-cookie-generation', 'http-cookie-path', 'http-cookie-share',
                   'http-ip-header', 'http-ip-header-name', 'http-multiplex',
                   'https-cookie-secure', 'id', 'ldb-method',
                   'mapped-addr', 'mappedip', 'mappedport',
                   'max-embryonic-connections', 'monitor', 'name',
                   'nat-source-vip', 'outlook-web-access', 'persistence',
                   'portforward', 'portmapping-type', 'protocol',
                   'realservers', 'server-type', 'service',
                   'src-filter', 'srcintf-filter', 'ssl-algorithm',
                   'ssl-certificate', 'ssl-cipher-suites', 'ssl-client-fallback',
                   'ssl-client-renegotiation', 'ssl-client-session-state-max', 'ssl-client-session-state-timeout',
                   'ssl-client-session-state-type', 'ssl-dh-bits', 'ssl-hpkp',
                   'ssl-hpkp-age', 'ssl-hpkp-backup', 'ssl-hpkp-include-subdomains',
                   'ssl-hpkp-primary', 'ssl-hpkp-report-uri', 'ssl-hsts',
                   'ssl-hsts-age', 'ssl-hsts-include-subdomains', 'ssl-http-location-conversion',
                   'ssl-http-match-host', 'ssl-max-version', 'ssl-min-version',
                   'ssl-mode', 'ssl-pfs', 'ssl-send-empty-frags',
                   'ssl-server-algorithm', 'ssl-server-cipher-suites', 'ssl-server-max-version',
                   'ssl-server-min-version', 'ssl-server-session-state-max', 'ssl-server-session-state-timeout',
                   'ssl-server-session-state-type', 'type', 'uuid',
                   'weblogic-server', 'websphere-server']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def firewall_vip(data, fos):
    vdom = data['vdom']
    firewall_vip_data = data['firewall_vip']
    filtered_data = filter_firewall_vip_data(firewall_vip_data)
    if firewall_vip_data['state'] == "present":
        return fos.set('firewall',
                       'vip',
                       data=filtered_data,
                       vdom=vdom)

    elif firewall_vip_data['state'] == "absent":
        return fos.delete('firewall',
                          'vip',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_firewall(data, fos):
    login(data)

    methodlist = ['firewall_vip']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "firewall_vip": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "arp-reply": {"required": False, "type": "str",
                              "choices": ["disable", "enable"]},
                "color": {"required": False, "type": "int"},
                "comment": {"required": False, "type": "str"},
                "dns-mapping-ttl": {"required": False, "type": "int"},
                "extaddr": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "extintf": {"required": False, "type": "str"},
                "extip": {"required": False, "type": "str"},
                "extport": {"required": False, "type": "str"},
                "gratuitous-arp-interval": {"required": False, "type": "int"},
                "http-cookie-age": {"required": False, "type": "int"},
                "http-cookie-domain": {"required": False, "type": "str"},
                "http-cookie-domain-from-host": {"required": False, "type": "str",
                                                 "choices": ["disable", "enable"]},
                "http-cookie-generation": {"required": False, "type": "int"},
                "http-cookie-path": {"required": False, "type": "str"},
                "http-cookie-share": {"required": False, "type": "str",
                                      "choices": ["disable", "same-ip"]},
                "http-ip-header": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "http-ip-header-name": {"required": False, "type": "str"},
                "http-multiplex": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "https-cookie-secure": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                "id": {"required": False, "type": "int"},
                "ldb-method": {"required": False, "type": "str",
                               "choices": ["static", "round-robin", "weighted",
                                           "least-session", "least-rtt", "first-alive",
                                           "http-host"]},
                "mapped-addr": {"required": False, "type": "str"},
                "mappedip": {"required": False, "type": "list",
                             "options": {
                                 "range": {"required": True, "type": "str"}
                             }},
                "mappedport": {"required": False, "type": "str"},
                "max-embryonic-connections": {"required": False, "type": "int"},
                "monitor": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "name": {"required": True, "type": "str"},
                "nat-source-vip": {"required": False, "type": "str",
                                   "choices": ["disable", "enable"]},
                "outlook-web-access": {"required": False, "type": "str",
                                       "choices": ["disable", "enable"]},
                "persistence": {"required": False, "type": "str",
                                "choices": ["none", "http-cookie", "ssl-session-id"]},
                "portforward": {"required": False, "type": "str",
                                "choices": ["disable", "enable"]},
                "portmapping-type": {"required": False, "type": "str",
                                     "choices": ["1-to-1", "m-to-n"]},
                "protocol": {"required": False, "type": "str",
                             "choices": ["tcp", "udp", "sctp",
                                         "icmp"]},
                "realservers": {"required": False, "type": "list",
                                "options": {
                                    "client-ip": {"required": False, "type": "str"},
                                    "healthcheck": {"required": False, "type": "str",
                                                    "choices": ["disable", "enable", "vip"]},
                                    "holddown-interval": {"required": False, "type": "int"},
                                    "http-host": {"required": False, "type": "str"},
                                    "id": {"required": True, "type": "int"},
                                    "ip": {"required": False, "type": "str"},
                                    "max-connections": {"required": False, "type": "int"},
                                    "monitor": {"required": False, "type": "str"},
                                    "port": {"required": False, "type": "int"},
                                    "status": {"required": False, "type": "str",
                                               "choices": ["active", "standby", "disable"]},
                                    "weight": {"required": False, "type": "int"}
                                }},
                "server-type": {"required": False, "type": "str",
                                "choices": ["http", "https", "imaps",
                                            "pop3s", "smtps", "ssl",
                                            "tcp", "udp", "ip"]},
                "service": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "src-filter": {"required": False, "type": "list",
                               "options": {
                                   "range": {"required": True, "type": "str"}
                               }},
                "srcintf-filter": {"required": False, "type": "list",
                                   "options": {
                                       "interface-name": {"required": True, "type": "str"}
                                   }},
                "ssl-algorithm": {"required": False, "type": "str",
                                  "choices": ["high", "medium", "low",
                                              "custom"]},
                "ssl-certificate": {"required": False, "type": "str"},
                "ssl-cipher-suites": {"required": False, "type": "list",
                                      "options": {
                                          "cipher": {"required": False, "type": "str",
                                                     "choices": ["TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                                                                 "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                                                                 "TLS-DHE-DSS-WITH-DES-CBC-SHA"]},
                                          "priority": {"required": True, "type": "int"},
                                          "versions": {"required": False, "type": "str",
                                                       "choices": ["ssl-3.0", "tls-1.0", "tls-1.1",
                                                                   "tls-1.2"]}
                                      }},
                "ssl-client-fallback": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                "ssl-client-renegotiation": {"required": False, "type": "str",
                                             "choices": ["allow", "deny", "secure"]},
                "ssl-client-session-state-max": {"required": False, "type": "int"},
                "ssl-client-session-state-timeout": {"required": False, "type": "int"},
                "ssl-client-session-state-type": {"required": False, "type": "str",
                                                  "choices": ["disable", "time", "count",
                                                              "both"]},
                "ssl-dh-bits": {"required": False, "type": "str",
                                "choices": ["768", "1024", "1536",
                                            "2048", "3072", "4096"]},
                "ssl-hpkp": {"required": False, "type": "str",
                             "choices": ["disable", "enable", "report-only"]},
                "ssl-hpkp-age": {"required": False, "type": "int"},
                "ssl-hpkp-backup": {"required": False, "type": "str"},
                "ssl-hpkp-include-subdomains": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                "ssl-hpkp-primary": {"required": False, "type": "str"},
                "ssl-hpkp-report-uri": {"required": False, "type": "str"},
                "ssl-hsts": {"required": False, "type": "str",
                             "choices": ["disable", "enable"]},
                "ssl-hsts-age": {"required": False, "type": "int"},
                "ssl-hsts-include-subdomains": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                "ssl-http-location-conversion": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                "ssl-http-match-host": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "ssl-max-version": {"required": False, "type": "str",
                                    "choices": ["ssl-3.0", "tls-1.0", "tls-1.1",
                                                "tls-1.2"]},
                "ssl-min-version": {"required": False, "type": "str",
                                    "choices": ["ssl-3.0", "tls-1.0", "tls-1.1",
                                                "tls-1.2"]},
                "ssl-mode": {"required": False, "type": "str",
                             "choices": ["half", "full"]},
                "ssl-pfs": {"required": False, "type": "str",
                            "choices": ["require", "deny", "allow"]},
                "ssl-send-empty-frags": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "ssl-server-algorithm": {"required": False, "type": "str",
                                         "choices": ["high", "medium", "low",
                                                     "custom", "client"]},
                "ssl-server-cipher-suites": {"required": False, "type": "list",
                                             "options": {
                                                 "cipher": {"required": False, "type": "str",
                                                            "choices": ["TLS-RSA-WITH-3DES-EDE-CBC-SHA",
                                                                        "TLS-DHE-RSA-WITH-DES-CBC-SHA",
                                                                        "TLS-DHE-DSS-WITH-DES-CBC-SHA"]},
                                                 "priority": {"required": True, "type": "int"},
                                                 "versions": {"required": False, "type": "str",
                                                              "choices": ["ssl-3.0", "tls-1.0", "tls-1.1",
                                                                          "tls-1.2"]}
                                             }},
                "ssl-server-max-version": {"required": False, "type": "str",
                                           "choices": ["ssl-3.0", "tls-1.0", "tls-1.1",
                                                       "tls-1.2", "client"]},
                "ssl-server-min-version": {"required": False, "type": "str",
                                           "choices": ["ssl-3.0", "tls-1.0", "tls-1.1",
                                                       "tls-1.2", "client"]},
                "ssl-server-session-state-max": {"required": False, "type": "int"},
                "ssl-server-session-state-timeout": {"required": False, "type": "int"},
                "ssl-server-session-state-type": {"required": False, "type": "str",
                                                  "choices": ["disable", "time", "count",
                                                              "both"]},
                "type": {"required": False, "type": "str",
                         "choices": ["static-nat", "load-balance", "server-load-balance",
                                     "dns-translation", "fqdn"]},
                "uuid": {"required": False, "type": "str"},
                "weblogic-server": {"required": False, "type": "str",
                                    "choices": ["disable", "enable"]},
                "websphere-server": {"required": False, "type": "str",
                                     "choices": ["disable", "enable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_firewall(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Provides an entry point for python scripts and python modules on the controller with the current python interpreter and optional code coverage collection."""

import os
import sys


def main():
    """Main entry point."""
    name = os.path.basename(__file__)
    args = [sys.executable]

    coverage_config = os.environ.get('COVERAGE_CONF')
    coverage_output = os.environ.get('COVERAGE_FILE')

    if coverage_config:
        if coverage_output:
            args += ['-m', 'coverage.__main__', 'run', '--rcfile', coverage_config]
        else:
            if sys.version_info >= (3, 4):
                import importlib.util

                found = bool(importlib.util.find_spec('coverage'))
            else:
                import imp

                try:
                    imp.find_module('coverage')
                    found = True
                except ImportError:
                    found = False

            if not found:
                exit('ERROR: Could not find `coverage` module. Did you use a virtualenv created without --system-site-packages or with the wrong interpreter?')

    if name == 'python.py':
        if sys.argv[1] == '-c':
            # prevent simple misuse of python.py with -c which does not work with coverage
            sys.exit('ERROR: Use `python -c` instead of `python.py -c` to avoid errors when code coverage is collected.')
    elif name == 'pytest':
        args += ['-m', 'pytest']
    else:
        args += [find_executable(name)]

    args += sys.argv[1:]

    os.execv(args[0], args)


def find_executable(name):
    """
    :type name: str
    :rtype: str
    """
    path = os.environ.get('PATH', os.path.defpath)
    seen = set([os.path.abspath(__file__)])

    for base in path.split(os.path.pathsep):
        candidate = os.path.abspath(os.path.join(base, name))

        if candidate in seen:
            continue

        seen.add(candidate)

        if os.path.exists(candidate) and os.access(candidate, os.F_OK | os.X_OK):
            return candidate

    raise Exception('Executable "%s" not found in path: %s' % (name, path))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2014, Ruggero Marchei <ruggero.marchei@daemonzone.net>
# Copyright: (c) 2015, Brian Coca <bcoca@ansible.com>
# Copyright: (c) 2016-2017, Konstantin Shalygin <k0ste@k0ste.ru>
# Copyright: (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['stableinterface'],
                    'supported_by': 'core'}

DOCUMENTATION = r'''
---
module: find
author: Brian Coca (@bcoca)
version_added: "2.0"
short_description: Return a list of files based on specific criteria
description:
    - Return a list of files based on specific criteria. Multiple criteria are AND'd together.
    - For Windows targets, use the M(win_find) module instead.
options:
    age:
        description:
            - Select files whose age is equal to or greater than the specified time.
            - Use a negative age to find files equal to or less than the specified time.
            - You can choose seconds, minutes, hours, days, or weeks by specifying the
              first letter of any of those words (e.g., "1w").
        type: str
    patterns:
        default: '*'
        description:
            - One or more (shell or regex) patterns, which type is controlled by C(use_regex) option.
            - The patterns restrict the list of files to be returned to those whose basenames match at
              least one of the patterns specified. Multiple patterns can be specified using a list.
            - This parameter expects a list, which can be either comma separated or YAML. If any of the
              patterns contain a comma, make sure to put them in a list to avoid splitting the patterns
              in undesirable ways.
        type: list
        aliases: [ pattern ]
    excludes:
        description:
            - One or more (shell or regex) patterns, which type is controlled by C(use_regex) option.
            - Items whose basenames match an C(excludes) pattern are culled from C(patterns) matches.
              Multiple patterns can be specified using a list.
        type: list
        aliases: [ exclude ]
        version_added: "2.5"
    contains:
        description:
            - One or more regex patterns which should be matched against the file content.
        type: str
    paths:
        description:
            - List of paths of directories to search. All paths must be fully qualified.
        type: list
        required: true
        aliases: [ name, path ]
    file_type:
        description:
            - Type of file to select.
            - The 'link' and 'any' choices were added in Ansible 2.3.
        type: str
        choices: [ any, directory, file, link ]
        default: file
    recurse:
        description:
            - If target is a directory, recursively descend into the directory looking for files.
        type: bool
        default: no
    size:
        description:
            - Select files whose size is equal to or greater than the specified size.
            - Use a negative size to find files equal to or less than the specified size.
            - Unqualified values are in bytes but b, k, m, g, and t can be appended to specify
              bytes, kilobytes, megabytes, gigabytes, and terabytes, respectively.
            - Size is not evaluated for directories.
    age_stamp:
        description:
            - Choose the file property against which we compare age.
        type: str
        choices: [ atime, ctime, mtime ]
        default: mtime
    hidden:
        description:
            - Set this to C(yes) to include hidden files, otherwise they will be ignored.
        type: bool
        default: no
    follow:
        description:
            - Set this to C(yes) to follow symlinks in path for systems with python 2.6+.
        type: bool
        default: no
    get_checksum:
        description:
            - Set this to C(yes) to retrieve a file's SHA1 checksum.
        type: bool
        default: no
    use_regex:
        description:
            - If C(no), the patterns are file globs (shell).
            - If C(yes), they are python regexes.
        type: bool
        default: no
    depth:
        description:
            - Set the maximum number of levels to decend into.
            - Setting recurse to C(no) will override this value, which is effectively depth 1.
            - Default is unlimited depth.
        type: int
        version_added: "2.6"
seealso:
- module: win_find
'''


EXAMPLES = r'''
- name: Recursively find /tmp files older than 2 days
  find:
    paths: /tmp
    age: 2d
    recurse: yes

- name: Recursively find /tmp files older than 4 weeks and equal or greater than 1 megabyte
  find:
    paths: /tmp
    age: 4w
    size: 1m
    recurse: yes

- name: Recursively find /var/tmp files with last access time greater than 3600 seconds
  find:
    paths: /var/tmp
    age: 3600
    age_stamp: atime
    recurse: yes

- name: Find /var/log files equal or greater than 10 megabytes ending with .old or .log.gz
  find:
    paths: /var/log
    patterns: '*.old,*.log.gz'
    size: 10m

# Note that YAML double quotes require escaping backslashes but yaml single quotes do not.
- name: Find /var/log files equal or greater than 10 megabytes ending with .old or .log.gz via regex
  find:
    paths: /var/log
    patterns: "^.*?\\.(?:old|log\\.gz)$"
    size: 10m
    use_regex: yes

- name: Find /var/log all directories, exclude nginx and mysql
  find:
    paths: /var/log
    recurse: no
    file_type: directory
    excludes: 'nginx,mysql'

# When using patterns that contain a comma, make sure they are formatted as lists to avoid splitting the pattern
- name: Use a single pattern that contains a comma formatted as a list
  find:
    paths: /var/log
    file_type: file
    use_regex: yes
    patterns: ['^_[0-9]{2,4}_.*.log$']

- name: Use multiple patterns that contain a comma formatted as a YAML list
  find:
    paths: /var/log
    file_type: file
    use_regex: yes
    patterns:
      - '^_[0-9]{2,4}_.*.log$'
      - '^[a-z]{1,5}_.*log$'

'''

RETURN = r'''
files:
    description: All matches found with the specified criteria (see stat module for full output of each dictionary)
    returned: success
    type: list
    sample: [
        { path: "/var/tmp/test1",
          mode: "0644",
          "...": "...",
          checksum: 16fac7be61a6e4591a33ef4b729c5c3302307523
        },
        { path: "/var/tmp/test2",
          "...": "..."
        },
        ]
matched:
    description: Number of matches
    returned: success
    type: int
    sample: 14
examined:
    description: Number of filesystem objects looked at
    returned: success
    type: int
    sample: 34
'''

import fnmatch
import grp
import os
import pwd
import re
import stat
import sys
import time

from ansible.module_utils.basic import AnsibleModule


def pfilter(f, patterns=None, excludes=None, use_regex=False):
    '''filter using glob patterns'''
    if patterns is None and excludes is None:
        return True

    if use_regex:
        if patterns and excludes is None:
            for p in patterns:
                r = re.compile(p)
                if r.match(f):
                    return True

        elif patterns and excludes:
            for p in patterns:
                r = re.compile(p)
                if r.match(f):
                    for e in excludes:
                        r = re.compile(e)
                        if r.match(f):
                            return False
                    return True

    else:
        if patterns and excludes is None:
            for p in patterns:
                if fnmatch.fnmatch(f, p):
                    return True

        elif patterns and excludes:
            for p in patterns:
                if fnmatch.fnmatch(f, p):
                    for e in excludes:
                        if fnmatch.fnmatch(f, e):
                            return False
                    return True

    return False


def agefilter(st, now, age, timestamp):
    '''filter files older than age'''
    if age is None:
        return True
    elif age >= 0 and now - st.__getattribute__("st_%s" % timestamp) >= abs(age):
        return True
    elif age < 0 and now - st.__getattribute__("st_%s" % timestamp) <= abs(age):
        return True
    return False


def sizefilter(st, size):
    '''filter files greater than size'''
    if size is None:
        return True
    elif size >= 0 and st.st_size >= abs(size):
        return True
    elif size < 0 and st.st_size <= abs(size):
        return True
    return False


def contentfilter(fsname, pattern):
    """
    Filter files which contain the given expression
    :arg fsname: Filename to scan for lines matching a pattern
    :arg pattern: Pattern to look for inside of line
    :rtype: bool
    :returns: True if one of the lines in fsname matches the pattern. Otherwise False
    """
    if pattern is None:
        return True

    prog = re.compile(pattern)

    try:
        with open(fsname) as f:
            for line in f:
                if prog.match(line):
                    return True

    except Exception:
        pass

    return False


def statinfo(st):
    pw_name = ""
    gr_name = ""

    try:  # user data
        pw_name = pwd.getpwuid(st.st_uid).pw_name
    except Exception:
        pass

    try:  # group data
        gr_name = grp.getgrgid(st.st_gid).gr_name
    except Exception:
        pass

    return {
        'mode': "%04o" % stat.S_IMODE(st.st_mode),
        'isdir': stat.S_ISDIR(st.st_mode),
        'ischr': stat.S_ISCHR(st.st_mode),
        'isblk': stat.S_ISBLK(st.st_mode),
        'isreg': stat.S_ISREG(st.st_mode),
        'isfifo': stat.S_ISFIFO(st.st_mode),
        'islnk': stat.S_ISLNK(st.st_mode),
        'issock': stat.S_ISSOCK(st.st_mode),
        'uid': st.st_uid,
        'gid': st.st_gid,
        'size': st.st_size,
        'inode': st.st_ino,
        'dev': st.st_dev,
        'nlink': st.st_nlink,
        'atime': st.st_atime,
        'mtime': st.st_mtime,
        'ctime': st.st_ctime,
        'gr_name': gr_name,
        'pw_name': pw_name,
        'wusr': bool(st.st_mode & stat.S_IWUSR),
        'rusr': bool(st.st_mode & stat.S_IRUSR),
        'xusr': bool(st.st_mode & stat.S_IXUSR),
        'wgrp': bool(st.st_mode & stat.S_IWGRP),
        'rgrp': bool(st.st_mode & stat.S_IRGRP),
        'xgrp': bool(st.st_mode & stat.S_IXGRP),
        'woth': bool(st.st_mode & stat.S_IWOTH),
        'roth': bool(st.st_mode & stat.S_IROTH),
        'xoth': bool(st.st_mode & stat.S_IXOTH),
        'isuid': bool(st.st_mode & stat.S_ISUID),
        'isgid': bool(st.st_mode & stat.S_ISGID),
    }


def main():
    module = AnsibleModule(
        argument_spec=dict(
            paths=dict(type='list', required=True, aliases=['name', 'path']),
            patterns=dict(type='list', default=['*'], aliases=['pattern']),
            excludes=dict(type='list', aliases=['exclude']),
            contains=dict(type='str'),
            file_type=dict(type='str', default="file", choices=['any', 'directory', 'file', 'link']),
            age=dict(type='str'),
            age_stamp=dict(type='str', default="mtime", choices=['atime', 'ctime', 'mtime']),
            size=dict(type='str'),
            recurse=dict(type='bool', default=False),
            hidden=dict(type='bool', default=False),
            follow=dict(type='bool', default=False),
            get_checksum=dict(type='bool', default=False),
            use_regex=dict(type='bool', default=False),
            depth=dict(type='int'),
        ),
        supports_check_mode=True,
    )

    params = module.params

    filelist = []

    if params['age'] is None:
        age = None
    else:
        # convert age to seconds:
        m = re.match(r"^(-?\d+)(s|m|h|d|w)?$", params['age'].lower())
        seconds_per_unit = {"s": 1, "m": 60, "h": 3600, "d": 86400, "w": 604800}
        if m:
            age = int(m.group(1)) * seconds_per_unit.get(m.group(2), 1)
        else:
            module.fail_json(age=params['age'], msg="failed to process age")

    if params['size'] is None:
        size = None
    else:
        # convert size to bytes:
        m = re.match(r"^(-?\d+)(b|k|m|g|t)?$", params['size'].lower())
        bytes_per_unit = {"b": 1, "k": 1024, "m": 1024**2, "g": 1024**3, "t": 1024**4}
        if m:
            size = int(m.group(1)) * bytes_per_unit.get(m.group(2), 1)
        else:
            module.fail_json(size=params['size'], msg="failed to process size")

    now = time.time()
    msg = ''
    looked = 0
    for npath in params['paths']:
        npath = os.path.expanduser(os.path.expandvars(npath))
        if os.path.isdir(npath):
            ''' ignore followlinks for python version < 2.6 '''
            for root, dirs, files in (sys.version_info < (2, 6, 0) and os.walk(npath)) or os.walk(npath, followlinks=params['follow']):
                if params['depth']:
                    depth = root.replace(npath.rstrip(os.path.sep), '').count(os.path.sep)
                    if files or dirs:
                        depth += 1
                    if depth > params['depth']:
                        del(dirs[:])
                        continue
                looked = looked + len(files) + len(dirs)
                for fsobj in (files + dirs):
                    fsname = os.path.normpath(os.path.join(root, fsobj))

                    if os.path.basename(fsname).startswith('.') and not params['hidden']:
                        continue

                    try:
                        st = os.lstat(fsname)
                    except Exception:
                        msg += "%s was skipped as it does not seem to be a valid file or it cannot be accessed\n" % fsname
                        continue

                    r = {'path': fsname}
                    if params['file_type'] == 'any':
                        if pfilter(fsobj, params['patterns'], params['excludes'], params['use_regex']) and agefilter(st, now, age, params['age_stamp']):

                            r.update(statinfo(st))
                            if stat.S_ISREG(st.st_mode) and params['get_checksum']:
                                r['checksum'] = module.sha1(fsname)
                            filelist.append(r)

                    elif stat.S_ISDIR(st.st_mode) and params['file_type'] == 'directory':
                        if pfilter(fsobj, params['patterns'], params['excludes'], params['use_regex']) and agefilter(st, now, age, params['age_stamp']):

                            r.update(statinfo(st))
                            filelist.append(r)

                    elif stat.S_ISREG(st.st_mode) and params['file_type'] == 'file':
                        if pfilter(fsobj, params['patterns'], params['excludes'], params['use_regex']) and \
                           agefilter(st, now, age, params['age_stamp']) and \
                           sizefilter(st, size) and contentfilter(fsname, params['contains']):

                            r.update(statinfo(st))
                            if params['get_checksum']:
                                r['checksum'] = module.sha1(fsname)
                            filelist.append(r)

                    elif stat.S_ISLNK(st.st_mode) and params['file_type'] == 'link':
                        if pfilter(fsobj, params['patterns'], params['excludes'], params['use_regex']) and agefilter(st, now, age, params['age_stamp']):

                            r.update(statinfo(st))
                            filelist.append(r)

                if not params['recurse']:
                    break
        else:
            msg += "%s was skipped as it does not seem to be a valid directory or it cannot be accessed\n" % npath

    matched = len(filelist)
    module.exit_json(files=filelist, changed=False, msg=msg, matched=matched, examined=looked)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import optparse
import re
from distutils.version import LooseVersion

import jinja2
import yaml
from jinja2 import Environment, FileSystemLoader

from ansible.module_utils._text import to_bytes
from ansible.playbook import Play
from ansible.playbook.block import Block
from ansible.playbook.role import Role
from ansible.playbook.task import Task
from ansible.utils._build_helpers import update_file_if_different

template_file = 'playbooks_keywords.rst.j2'
oblist = {}
clist = []
class_list = [Play, Role, Block, Task]

p = optparse.OptionParser(
    version='%prog 1.0',
    usage='usage: %prog [options]',
    description='Generate playbook keyword documentation from code and descriptions',
)
p.add_option("-T", "--template-dir", action="store", dest="template_dir", default="../templates", help="directory containing Jinja2 templates")
p.add_option("-o", "--output-dir", action="store", dest="output_dir", default='/tmp/', help="Output directory for rst files")
p.add_option("-d", "--docs-source", action="store", dest="docs", default=None, help="Source for attribute docs")

(options, args) = p.parse_args()

for aclass in class_list:
    aobj = aclass()
    name = type(aobj).__name__

    if options.docs:
        with open(options.docs) as f:
            docs = yaml.safe_load(f)
    else:
        docs = {}

    # build ordered list to loop over and dict with attributes
    clist.append(name)
    oblist[name] = dict((x, aobj.__dict__['_attributes'][x]) for x in aobj.__dict__['_attributes'] if 'private' not in x or not x.private)

    # pick up docs if they exist
    for a in oblist[name]:
        if a in docs:
            oblist[name][a] = docs[a]
        else:
            # check if there is an alias, otherwise undocumented
            alias = getattr(getattr(aobj, '_%s' % a), 'alias', None)
            if alias and alias in docs:
                oblist[name][alias] = docs[alias]
                del oblist[name][a]
            else:
                oblist[name][a] = ' UNDOCUMENTED!! '

    # loop is really with_ for users
    if name == 'Task':
        oblist[name]['with_<lookup_plugin>'] = 'The same as ``loop`` but magically adds the output of any lookup plugin to generate the item list.'

    # local_action is implicit with action
    if 'action' in oblist[name]:
        oblist[name]['local_action'] = 'Same as action but also implies ``delegate_to: localhost``'

    # remove unusable (used to be private?)
    for nouse in ('loop_args', 'loop_with'):
        if nouse in oblist[name]:
            del oblist[name][nouse]

env = Environment(loader=FileSystemLoader(options.template_dir), trim_blocks=True,)
template = env.get_template(template_file)
outputname = options.output_dir + template_file.replace('.j2', '')
tempvars = {'oblist': oblist, 'clist': clist}

keyword_page = template.render(tempvars)
if LooseVersion(jinja2.__version__) < LooseVersion('2.10'):
    # jinja2 < 2.10's indent filter indents blank lines.  Cleanup
    keyword_page = re.sub(' +\n', '\n', keyword_page)

update_file_if_different(outputname, to_bytes(keyword_page))



# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Fran Fitzpatrick <francis.x.fitzpatrick@gmail.com> fxfitz
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from mock import call, MagicMock
import pytest

from ansible.errors import AnsibleConnectionFailure
from ansible.plugins.terminal import junos


@pytest.fixture
def junos_terminal():
    mock_connection = MagicMock()
    return junos.TerminalModule(mock_connection)


def test_on_open_shell_sets_terminal_parameters(junos_terminal):
    expected_calls = [
        call(b'set cli timestamp disable'),
        call(b'set cli screen-length 0'),
        call(b'set cli screen-width 1024'),
    ]
    junos_terminal._exec_cli_command = MagicMock()
    junos_terminal._get_prompt = MagicMock()

    junos_terminal._get_prompt.return_value = b'user@localhost >'
    junos_terminal.on_open_shell()
    junos_terminal._exec_cli_command.assert_has_calls(expected_calls)


def test_on_open_shell_enters_cli_if_root_prompt(junos_terminal):
    expected_calls = [
        call(b'cli'),
        call(b'set cli timestamp disable'),
        call(b'set cli screen-length 0'),
        call(b'set cli screen-width 1024'),
    ]
    junos_terminal._exec_cli_command = MagicMock()
    junos_terminal._get_prompt = MagicMock()

    junos_terminal._connection.get_prompt.return_value = b'root@localhost%'
    junos_terminal.on_open_shell()
    junos_terminal._exec_cli_command.assert_has_calls(expected_calls)


def test_on_open_shell_raises_problem_setting_terminal_config(junos_terminal):
    junos_terminal._connection.exec_command.side_effect = AnsibleConnectionFailure
    with pytest.raises(AnsibleConnectionFailure) as exc:
        junos_terminal.on_open_shell()

    assert 'unable to set terminal parameters' in str(exc.value)



# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
    name: linode
    plugin_type: inventory
    author:
      - Luke Murphy (@lwm)
    short_description: Ansible dynamic inventory plugin for Linode.
    version_added: "2.8"
    requirements:
        - python >= 2.7
        - linode_api4 >= 2.0.0
    description:
        - Reads inventories from the Linode API v4.
        - Uses a YAML configuration file that ends with linode.(yml|yaml).
        - Linode labels are used by default as the hostnames.
        - The inventory groups are built from groups and not tags.
    options:
        plugin:
            description: marks this as an instance of the 'linode' plugin
            required: true
            choices: ['linode']
        access_token:
            description: The Linode account personal access token.
            required: true
            env:
                - name: LINODE_ACCESS_TOKEN
        regions:
          description: Populate inventory with instances in this region.
          default: []
          type: list
          required: false
        types:
          description: Populate inventory with instances with this type.
          default: []
          type: list
          required: false
'''

EXAMPLES = r'''
# Minimal example. `LINODE_ACCESS_TOKEN` is exposed in environment.
plugin: linode

# Example with regions, types, groups and access token
plugin: linode
access_token: foobar
regions:
  - eu-west
types:
  - g5-standard-2
'''

import os

from ansible.errors import AnsibleError, AnsibleParserError
from ansible.module_utils.six import string_types
from ansible.plugins.inventory import BaseInventoryPlugin


try:
    from linode_api4 import LinodeClient
    from linode_api4.errors import ApiError as LinodeApiError
except ImportError:
    raise AnsibleError('the Linode dynamic inventory plugin requires linode_api4.')


class InventoryModule(BaseInventoryPlugin):

    NAME = 'linode'

    def _build_client(self):
        """Build the Linode client."""

        access_token = self.get_option('access_token')

        if access_token is None:
            try:
                access_token = os.environ['LINODE_ACCESS_TOKEN']
            except KeyError:
                pass

        if access_token is None:
            raise AnsibleError((
                'Could not retrieve Linode access token '
                'from plugin configuration or environment'
            ))

        self.client = LinodeClient(access_token)

    def _get_instances_inventory(self):
        """Retrieve Linode instance information from cloud inventory."""
        try:
            self.instances = self.client.linode.instances()
        except LinodeApiError as exception:
            raise AnsibleError('Linode client raised: %s' % exception)

    def _add_groups(self):
        """Add Linode instance groups to the dynamic inventory."""
        self.linode_groups = set(
            filter(None, [
                instance.group
                for instance
                in self.instances
            ])
        )

        for linode_group in self.linode_groups:
            self.inventory.add_group(linode_group)

    def _filter_by_config(self, regions, types):
        """Filter instances by user specified configuration."""
        if regions:
            self.instances = [
                instance for instance in self.instances
                if instance.region.id in regions
            ]

        if types:
            self.instances = [
                instance for instance in self.instances
                if instance.type.id in types
            ]

    def _add_instances_to_groups(self):
        """Add instance names to their dynamic inventory groups."""
        for instance in self.instances:
            self.inventory.add_host(instance.label, group=instance.group)

    def _add_hostvars_for_instances(self):
        """Add hostvars for instances in the dynamic inventory."""
        for instance in self.instances:
            hostvars = instance._raw_json
            for hostvar_key in hostvars:
                self.inventory.set_variable(
                    instance.label,
                    hostvar_key,
                    hostvars[hostvar_key]
                )

    def _validate_option(self, name, desired_type, option_value):
        """Validate user specified configuration data against types."""
        if isinstance(option_value, string_types) and desired_type == list:
            option_value = [option_value]

        if option_value is None:
            option_value = desired_type()

        if not isinstance(option_value, desired_type):
            raise AnsibleParserError(
                'The option %s (%s) must be a %s' % (
                    name, option_value, desired_type
                )
            )

        return option_value

    def _get_query_options(self, config_data):
        """Get user specified query options from the configuration."""
        options = {
            'regions': {
                'type_to_be': list,
                'value': config_data.get('regions', [])
            },
            'types': {
                'type_to_be': list,
                'value': config_data.get('types', [])
            },
        }

        for name in options:
            options[name]['value'] = self._validate_option(
                name,
                options[name]['type_to_be'],
                options[name]['value']
            )

        regions = options['regions']['value']
        types = options['types']['value']

        return regions, types

    def verify_file(self, path):
        """Verify the Linode configuration file."""
        if super(InventoryModule, self).verify_file(path):
            endings = ('linode.yaml', 'linode.yml')
            if any((path.endswith(ending) for ending in endings)):
                return True
        return False

    def parse(self, inventory, loader, path, cache=True):
        """Dynamically parse Linode the cloud inventory."""
        super(InventoryModule, self).parse(inventory, loader, path)

        self._build_client()

        self._get_instances_inventory()

        config_data = self._read_config_data(path)
        regions, types = self._get_query_options(config_data)
        self._filter_by_config(regions, types)

        self._add_groups()
        self._add_instances_to_groups()
        self._add_hostvars_for_instances()



# -*- coding: utf-8 -*-
# Copyright (c) 2019 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import pytest

from ansible.module_utils.common.parameters import list_deprecations


@pytest.fixture
def params():
    return {
        'name': 'bob',
        'dest': '/etc/hosts',
        'state': 'present',
        'value': 5,
    }


def test_list_deprecations():
    argument_spec = {
        'old': {'type': 'str', 'removed_in_version': '2.5'}
    }

    params = {
        'name': 'rod',
        'old': 'option',
    }
    result = list_deprecations(argument_spec, params)
    for item in result:
        assert item['msg'] == "Param 'old' is deprecated. See the module docs for more information"
        assert item['version'] == '2.5'



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit tests ONTAP Ansible module: na_ontap_software_update '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_software_update \
    import NetAppONTAPSoftwareUpdate as my_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, parm1=None, parm2=None):
        ''' save arguments '''
        self.type = kind
        self.parm1 = parm1
        self.parm2 = parm2
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.type == 'software_update':
            xml = self.build_software_update_info(self.parm1, self.parm2)
        self.xml_out = xml
        return xml

    def autosupport_log(self):
        ''' mock autosupport log'''
        return None

    @staticmethod
    def build_software_update_info(status, node):
        ''' build xml data for software-update-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        data = {
            'num-records': 1,
            'attributes-list': {'cluster-image-info': {'node-id': node}},
            'progress-status': status,
            'attributes': {'ndu-progress-info': {'overall-status': 'completed',
                                                 'completed-node-count': '0'}},
        }
        xml.translate_struct(data)
        print(xml.to_string())
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()
        self.use_vsim = False

    def set_default_args(self):
        if self.use_vsim:
            hostname = '10.10.10.10'
            username = 'admin'
            password = 'admin'
            node = 'vsim1'
            package_version = 'Fattire__9.3.0'
            package_url = 'abc.com'
        else:
            hostname = 'hostname'
            username = 'username'
            password = 'password'
            node = 'abc'
            package_version = 'test'
            package_url = 'abc.com'
        return dict({
            'hostname': hostname,
            'username': username,
            'password': password,
            'nodes': node,
            'package_version': package_version,
            'package_url': package_url,
            'https': 'true'
        })

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            my_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_ensure_image_get_called(self):
        ''' a more interesting test '''
        set_module_args(self.set_default_args())
        my_obj = my_module()
        my_obj.server = self.server
        cluster_image_get = my_obj.cluster_image_get()
        print('Info: test_software_update_get: %s' % repr(cluster_image_get))
        assert cluster_image_get is None

    def test_ensure_apply_for_update_called(self):
        ''' updating software and checking idempotency '''
        module_args = {}
        module_args.update(self.set_default_args())
        module_args.update({'package_url': 'abc.com'})
        set_module_args(module_args)
        my_obj = my_module()
        my_obj.autosupport_log = Mock(return_value=None)
        if not self.use_vsim:
            my_obj.server = self.server
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Info: test_software_update_apply: %s' % repr(exc.value))
        assert not exc.value.args[0]['changed']
        if not self.use_vsim:
            my_obj.server = MockONTAPConnection('software_update', 'async_pkg_get_phase_complete', 'abc')
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Info: test_software_update_apply: %s' % repr(exc.value))
        assert exc.value.args[0]['changed']
